Package: bumphunter
Version: 1.55.1
Title: Bump Hunter
Description: Tools for finding bumps in genomic data
Authors@R: c(person(c("Rafael", "A."), "Irizarry", role = "aut", email
    = "rafa@ds.dfci.harvard.edu"), person("Martin", "Aryee", role = "aut"),
    person(c("Kasper", "Daniel"), "Hansen", role = "aut"), person(c("Hector",
    "Corrada"), "Bravo", role = "aut"), person("Shan", "Andrews", role =
    "ctb"), person(c("Andrew", "E."), "Jaffe", role = "ctb"), person("Harris",
    "Jaffee", role = "ctb"), person("Leonardo", "Collado-Torres", role="ctb"),
    person("Tamilselvi", "Guharaj", role = "cre", email =
    "selvi@ds.dfci.harvard.edu"))
Depends: R (>= 3.5), S4Vectors (>= 0.9.25), IRanges (>= 2.3.23),
        Seqinfo, GenomicRanges, foreach, iterators, methods, parallel,
        locfit
Suggests: testthat, RUnit, doParallel, GenomeInfoDb, txdbmaker,
        org.Hs.eg.db, TxDb.Hsapiens.UCSC.hg19.knownGene
Imports: matrixStats, limma, doRNG, BiocGenerics, utils,
        GenomicFeatures, AnnotationDbi, stats
License: Artistic-2.0
LazyData: yes
URL: https://github.com/rafalab/bumphunter
biocViews: DNAMethylation, Epigenetics, Infrastructure,
        MultipleComparison, ImmunoOncology
RoxygenNote: 6.1.1
Config/pak/sysreqs: make libbz2-dev liblzma-dev libpng-dev libxml2-dev
        libssl-dev xz-utils zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-05-25 21:15:24 UTC
RemoteUrl: https://github.com/bioc/bumphunter
RemoteRef: HEAD
RemoteSha: 51b589f2c3be64859df7b50431005b8cec3b72d7
NeedsCompilation: no
Packaged: 2026-05-26 07:47:51 UTC; root
Author: Rafael A. Irizarry [aut],
  Martin Aryee [aut],
  Kasper Daniel Hansen [aut],
  Hector Corrada Bravo [aut],
  Shan Andrews [ctb],
  Andrew E. Jaffe [ctb],
  Harris Jaffee [ctb],
  Leonardo Collado-Torres [ctb],
  Tamilselvi Guharaj [cre]
Maintainer: Tamilselvi Guharaj <selvi@ds.dfci.harvard.edu>
Built: R 4.6.0; ; 2026-05-26 07:53:08 UTC; unix

importFrom(foreach, "%dopar%", foreach, getDoParName, getDoParRegistered,
           getDoParWorkers, registerDoSEQ, getDoParVersion)
importFrom(iterators, iter)
importFrom(stats, model.matrix, preplot, rnorm)
importFrom(doRNG, "%dorng%")
importFrom(methods, is)
importClassesFrom(methods, matrix)
import(matrixStats)
import(BiocGenerics)
import(S4Vectors)
importFrom(IRanges,PartitioningByEnd)
importMethodsFrom(IRanges, as.list, as.matrix,
                  diff, distance, distanceToNearest,
                  end, mean,
                  median, nearest, quantile, ranges, reduce,
                  resize, runmed, start,
                  which, which.min, width)
importFrom(IRanges, CharacterList, IRanges)
importFrom(Seqinfo, seqnames)
importFrom(GenomicRanges, GRanges, makeGRangesFromDataFrame)
importMethodsFrom(GenomicRanges, sort)
importFrom(locfit, locfit)
importFrom(locfit, lp)
importFrom(utils, packageDescription)
importFrom(parallel, mclapply, stopCluster)
importFrom(GenomicFeatures, exonsBy, cdsBy, transcriptsBy)
importFrom(AnnotationDbi, mappedkeys)
importMethodsFrom(AnnotationDbi, mapIds)

export(bumphunter, bumphunterEngine, clusterMaker, boundedClusterMaker,
       dummyData, getSegments, 
       regionFinder,
       annotateNearest, annotateTranscripts, matchGenes,
       loessByCluster, runmedByCluster, smoother,
       locfitByCluster)
exportMethods(bumphunter)
S3method(print,bumps)

This is the developer version of the [bumphunter](http://bioconductor.org/packages/devel/bioc/html/bumphunter.html) R/Bioconductor package. 

[Bioconductor build/check report](http://master.bioconductor.org/checkResults/devel/bioc-LATEST/bumphunter/)

library(bumphunter)
  
## Generate dummy data
set.seed(123)
dat <- dummyData()

## loessByCluster      
loessSmoothed <- loessByCluster(cluster=dat$cluster, y=dat$mat[,1],
                              bpSpan = 1000, minNum=7, minInSpan=5, maxSpan=1)
save(loessSmoothed, file="../unitTests/loessSmoothed.rda")

## runmedByCluster
runmedSmoothed <- runmedByCluster(cluster=dat$cluster, y=dat$mat[,1],  k=5, endrule="constant")
save(runmedSmoothed, file="../unitTests/runmedSmoothed.rda")

## bumphunter test 1
set.seed(123)
bumpsTest1 <- bumphunter (dat$mat, design=dat$design, chr=dat$chr, pos=dat$pos, cluster=dat$cluster,
                          coef=2, cutoff=0.28, smooth=TRUE, B=500,
                          verbose=TRUE, smoothFunction=loessByCluster)
save(bumpsTest1, file="../unitTests/bumpsTest1.rda")

## bumphunter test 2: chr unspecified
set.seed(123)
bumpsTest2 <- bumphunter (dat$mat, design=dat$design, pos=dat$pos, cluster=dat$cluster,
                          coef=2, cutoff=0.28, maxGap=500, smooth=TRUE, B=500,
                          verbose=TRUE, smoothFunction=loessByCluster)
save(bumpsTest2, file="../unitTests/bumpsTest2.rda")

## Generate dummy data that can be used for unit tests and simple examples

dummyData <- function(n1=5, n2=5, sd=0.2, l=100, spacing=100, clusterSpacing=1e5, numClusters=5) {
    ## removed a set seed
    cluster <- rep(1:numClusters, each=l/numClusters)  
    chr <- c(rep(1, l*.8), rep(2, l*.2))
    pos <- cluster*clusterSpacing + seq(0, by=spacing, length=l) + sample(l/10, l, replace=TRUE)
    ## Prepare data
    x <- c(rep(0, 2*l/numClusters), seq(0, 1, length.out=l/numClusters) * 2 * pi, rep(0, 2*l/numClusters))
    grp1 <- replicate(n1, sin(x)) + rnorm(n1*l, sd=sd)
    grp2 <- replicate(n2, rep(0,l)) + rnorm(n1*l, sd=sd)
    mat <- cbind(grp1, grp2)
    dd <- data.frame(x=factor(c(rep(1, n1), rep(2,n2))))
    design <- model.matrix(~ x, dd, contrasts = list(x="contr.helmert"))
    return(list(mat=mat, design=design, chr=chr, pos=pos, cluster=cluster, n1=n1, n2=n2))
}

test_bumphunter <- function () {
    ## Generate dummy data
    set.seed(123)
    dat <- dummyData()
    
    ## Test 1
    set.seed(123)
    bumps <- bumphunter (dat$mat, design=dat$design, chr=dat$chr, pos=dat$pos,
                         cluster=dat$cluster, coef=2, cutoff=0.28,
                         smooth=TRUE, B=500, verbose=TRUE, smoothFunction=loessByCluster)
    load(file.path(path.package("bumphunter"), "unitTests", "bumpsTest1.rda"))
    nam <- setdiff(names(bumps), "algorithm")
    checkEquals(bumps[nam], bumpsTest1[nam])
    
    ## Test 2: chr unspecified
    set.seed(123)
    bumps <- bumphunter (dat$mat, design=dat$design, pos=dat$pos,
                         cluster=dat$cluster, coef=2, cutoff=0.28,
                         maxGap=500, smooth=TRUE, B=500, verbose=TRUE, smoothFunction=loessByCluster)
    load(file.path(path.package("bumphunter"), "unitTests", "bumpsTest2.rda"))
    nam <- setdiff(names(bumps), "algorithm")
    checkEquals(bumps[nam], bumpsTest2[nam])
}

test_bumphunterParallel <- function() {
    set.seed(123)
    dat <- dummyData()

    if(require(doParallel)) {
        registerDoParallel(cores = 2)
        set.seed(123)
        bumps <- bumphunter (dat$mat, design=dat$design, chr=dat$chr, pos=dat$pos,
                             cluster=dat$cluster, coef=2, cutoff=0.28,
                             smooth=TRUE, B=500, verbose=TRUE, smoothFunction=loessByCluster)
        load(file.path(path.package("bumphunter"), "unitTests", "bumpsTest1.rda"))
        nam <- setdiff(names(bumps), "algorithm")
        checkEquals(bumps[nam], bumpsTest1[nam])
        cl <- makeCluster(2)
        registerDoParallel(cl = cl)
        set.seed(123)
        bumps <- bumphunter (dat$mat, design=dat$design, chr=dat$chr, pos=dat$pos,
                             cluster=dat$cluster, coef=2, cutoff=0.28,
                             smooth=TRUE, B=500, verbose=TRUE, smoothFunction=loessByCluster)
        load(file.path(path.package("bumphunter"), "unitTests", "bumpsTest1.rda"))
        nam <- setdiff(names(bumps), "algorithm")
        checkEquals(bumps[nam], bumpsTest1[nam])
    }
}
    


    

annotateNearest <- function(x, subject, annotate=TRUE, ...) {

    if (is.data.frame(x)) x <- makeGRangesFromDataFrame(x)
    
    if(is.data.frame(subject)) subject <- makeGRangesFromDataFrame(subject)
    
    if(class(x)!="GRanges" | class(subject)!="GRanges")
        stop("x and subject must be GRanges or a data.frame with column names chr, start and end.")
    
    dots <- list(...)
    names.dots <- names(dots)
    if ("select" %in% names.dots && dots$select == "all")
	stop("select=\"all\" not supported (yet)")
    
    # find nearests
    NN <- smarterNearest(x, subject, ...)
    nearest_exists <- !is.na(NN)

    # full return value
    Ret <- matrix(NA, nrow=length(NN), ncol=7)
    colnames(Ret) <- c("distance", "subjectHits", "type", "amountOverlap",
	"insideDist", "size1", "size2")
    Ret <- data.frame(Ret, stringsAsFactors=FALSE)

    # queries with a nearest target
    x <- x[nearest_exists]
    # corresponding nearest targets
    y <- subject[NN[nearest_exists],]

    # partial value for when nearest exists
    ret <- matrix(NA, nrow=length(x), ncol=7)
    colnames(ret) <- c("distance", "subjectHits", "type", "amountOverlap",
	"insideDistance", "size1", "size2")
    ret[,"subjectHits"] <- NN[nearest_exists]
    ret <- data.frame(ret, stringsAsFactors=FALSE)

    x.start <- start(x)
    y.start <- start(y)
    x.end <- end(x)
    y.end <- end(y)

    ret$type[(x.start > y.start & x.end <= y.end) |
             (x.start >= y.start & x.end < y.end)] <- "inside"
    ret$type[x.start <= y.start & x.end >= y.end] <- "cover"
    ret$type[x.start > y.end] <- "disjointR"
    ret$type[x.end < y.start] <- "disjointL"
    ret$type[x.start > y.start & x.start <= y.end & x.end > y.end] <- "overlapR"    
    ret$type[x.start < y.start & x.end >= y.start & x.end < y.end] <- "overlapL"
    ret$distance <- 0L

    i <- ret$type == "disjointR"
    ret$distance[i] <- y.end[i] - x.start[i]

    i <- ret$type == "disjointL"
    ret$distance[i] <- y.start[i] - x.end[i]

    if (!annotate) {
	Ret[nearest_exists,] <- ret
	return(as.matrix(Ret[, c("distance", "subjectHits")]))
    }

    i <- ret$type == "overlapR"
    ret$amountOverlap[i] <- -1L * (y.end[i] - x.start[i] + 1L)

    i <- ret$type == "overlapL"
    ret$amountOverlap[i] <- x.end[i] - y.start[i] + 1L

    ret$type[ret$type%in%c("disjointR", "disjointL")] <- "disjoint"
    ret$type[ret$type%in%c("overlapR", "overlapL")] <- "overlap"

    ## insideDistance column:
    i <- ret$type == "inside"	#no missing ret$type at this point
    tmp0 <- cbind(x.end[i] - y.end[i], x.start[i] - y.start[i])
    tmp <- apply(abs(tmp0), 1, which.min)
    tmpinsidedist <- tmp0[, 1]
    tmpIndex <- tmp == 2
    tmpinsidedist[tmpIndex] <- tmp0[tmpIndex, 2]
    ret$insideDistance[i] <- tmpinsidedist

    ## size1 and size2 columns:
    ret$size1 <- x.end - x.start + 1L
    ret$size2 <- y.end - y.start + 1L
    
    Ret[nearest_exists,] <- ret
    Ret
}


#######################################################
#######################################################

annotateTranscripts <-function(txdb, annotationPackage=NULL, by=c("tx","gene"),codingOnly=FALSE,verbose=TRUE,requireAnnotation=FALSE, mappingInfo = NULL, simplifyGeneID = FALSE){

    if( class(txdb)!="TxDb") stop("txdb must be of class TxDb")

    if(is.null(annotationPackage)){
        organism <- organism(txdb)
        organism <- strsplit(organism," ")[[1]]
        organism <- paste0(substr(organism[1],1,1),
                           tolower(substr(organism[2],1,1)))
        annotationPackage <-  paste("org",organism,"eg.db",sep=".")
        if(verbose)
            message(sprintf("No annotationPackage supplied. Trying %s.",annotationPackage))
    }  

    if(!require(annotationPackage,character.only=TRUE)){
        if(requireAnnotation){
            stop("Can't load ",annotationPackage,".\nMake sure library is installed.\nAnd make sure the organism argument follows the convention here http://www.bioconductor.org/packages/release/data/annotation/.\nFor example for human use Hs")}else{
                message("Could not load ",annotationPackage,". Will continue without annotation")
                annotationPackage <- NULL
            }
    }
    
##################################################
### Get TSS and TSE
#######################################################

    if(verbose) message("Getting TSS and TSE.")
    
    by <- match.arg(by)
        
    if(by=="tx"){
        tt <- transcriptsBy(txdb, by="gene")
        seqinfo <- seqinfo(tt)
        ## the transcript names
        LL <- tt@unlistData@elementMetadata@listData
        txx <- unlist(tt)$tx_name
    } else{
        tt <- genes(txdb)
        seqinfo <- seqinfo(tt)
        txx <- names(tt)
    }
    
    
    chr <- unlist(CharacterList(seqnames(tt)))
    strand <- unlist(CharacterList(strand(tt)))
    
    RR <- ranges(tt)
    ## Useful for switching Gencode IDs to Ensembl IDs
    if(simplifyGeneID) names(RR) <- gsub('\\..*', '', names(RR))
    geneid <- names(RR)
    ## starts / ends (left/right endpoints):
    TSS <- unlist(start(RR))
    TSE <- unlist(end(RR))
    
    
######################################################
### Get CSS and CSE
#######################################################
    
    if(verbose) message("Getting CSS and CSE.")

    cds <- cdsBy(txdb, by=by, use.names=(by=="tx"))
    with_coding <- which(txx %in% names(cds))
    
    ## take only the left- and right-most endpts:
    cds_ranges <- range(cds[txx[with_coding]])
    
    ## starts and ends (really left / right endpts):
    CSS <- CSE <- integer(length(txx))
    CSS[with_coding] <- unlist(start(cds_ranges))
    CSE[with_coding] <- unlist(end(cds_ranges))
    CSS[-with_coding] <- NA
    CSE[-with_coding] <- NA
    
    ##################################################
    ### Get Exons
    #######################################################

    if(verbose) message("Getting exons.")

    ee <- exonsBy(txdb, by=by, use.names=(by=="tx"))
    ## exon groups of interest in transcript order,
    ## and sorted "left to right" within each group
    Exons <- reduce(ranges(ee)[txx])
    Nexons <- elementNROWS(Exons)

    ##now annotate genes
    if(by=="tx") TT <- elementNROWS(tt) else TT <- rep(1,length(tt))
    Geneid <- Rle(geneid, TT)

    Tx <- txx

    if(!is.null(annotationPackage)){
        if(verbose) message("Annotating genes.")
            
        if(!is.null(mappingInfo)) {
            stopifnot(all(c('column', 'keytype', 'multiVals') %in% names(mappingInfo)))
            geneid <- mapIds(get(annotationPackage), keys = geneid, column= mappingInfo$column, keytype = mappingInfo$keytype, multiVals = mappingInfo$multiVals)
        }
        

        ##Annotate transcrtipt
        ##cant use select cause map is not 1-1 
        map <- get(gsub("\\.db", "SYMBOL", annotationPackage))
        which <- mappedkeys(map)
        symbols <- sapply(as.list(map[which]), paste, collapse=" ")
        genes <- symbols[geneid]	
        
        map <- get(gsub("\\.db", "REFSEQ", annotationPackage))
        which <- mappedkeys(map)
        symbols <- sapply(as.list(map[which]), paste, collapse=" ")
        refseq <- symbols[geneid]	# a handful of NA
        
        Gene <- Rle(genes, TT)
        Refseq <- Rle(refseq, TT)
    } else {
        Gene <- Rle(NA, sum(TT))
        Refseq <- Rle(NA, sum(TT))
    }

    transcripts=GRanges(ranges=IRanges(start=TSS, end=TSE),
        seqnames=chr, strand=strand,
        seqinfo=seqinfo,
        CSS, CSE, Tx, Geneid, Gene, Refseq, Nexons, Exons)
    if(codingOnly) transcripts <- transcripts[!is.na(values(transcripts)$CSS),]
    attributes(transcripts)$description <- "annotatedTranscripts"
    return(transcripts)
}

matchGenes <- function(x,subject, type=c("any","fiveprime"),
                       promoterDist=2500,
                       skipExons=FALSE,
                       verbose=TRUE){

    if(attributes(subject)$description!="annotatedTranscripts")
        stop("subject must be the output of function annotateTranscripts or have attribute(subject)$description=\"annotatedTranscripts\".")
    
    if (is.data.frame(x)) x <- makeGRangesFromDataFrame(x)
    if(class(x)!="GRanges") stop("x must be GRanges or a data.frame with column names chr, start and end.")

    type=match.arg(type)
    
    if(type=="fiveprime"){
        map <- smarterNearest(x,resize(subject,width=1))
    } else{
        map <- smarterNearest(x,subject)
    }
    
    ind <- which(!is.na(map))
#    dist <- rep(NA,length(map))
#    dist[ind] <- distance(x[ind,],subject[map[ind],])
#    
#    dist[ind] <- dist[ind]*ifelse(
#        (strand(subject[map[ind],])=="+" &
#         end(subject[map[ind],]) < start(x[ind,])) |
#        (strand(subject[map[ind],])=="-" &
#         start(subject[map[ind],]) > end(x[ind,])),-1,1)
    
    type=rep("",length(x))
    subtype=rep("",length(x))
    ctype=rep("",length(x))
    dist=rep(0,length(x))	# distance to 5' end of the gene
    insidedistance<-rep(NA,length(x))
    exonnumber<-rep(NA,length(x))
    nexons <- rep(NA,length(x))
    geneL=rep(0,length(x))
    codingL=rep(0,length(x))
    subdist=rep(0,length(x))
    genenames=rep("",length(x))
    geneannotation=rep("",length(x))
    geneid=rep("",length(x))
    strands=rep("",length(x))

    
    genenames[ind]<-as.character(values(subject[map[ind],])$Gene)
    geneannotation[ind]<-as.character(values(subject[map[ind],])$Refseq)
    geneid[ind]<-as.character(values(subject[map[ind],])$Geneid)
    nexons[ind] <- values(subject[map[ind],])$Nexons
    strands[ind] <- as.character(strand(subject[map[ind],]))
    
    ## This loop could be moved to C. Note: it uses nearest
    for(j in ind){
        
        i <- map[ind][j]
        
        if(verbose & j%%100==0) cat(".")
        
        TS = start(subject)[i]
        TE = end(subject)[i]
        geneL[j] = TE-TS
        if(!is.na(subject$CSS[i])){
            CS = subject$CSS[i]
            CE = subject$CSE[i]
            codingL[j]=CE-CS
        } else {
            CS <- CE <- codingL <- NA
        }
        exons <- subject[i,]$Exons[[1]]
        Exons <- cbind(start(exons), end(exons))
        
        Strand= ifelse(strand(subject[i,])=="+",1,-1)
        S = start(x[j,])
        E = end(x[j,])
        
#        type[j]=""
        
        if(S <= TS & E >= TE){
            type[j]="covers"
            subtype[j]="covers exon(s)"
            
        } else{
            if(E < TS){
                if(Strand==1){
                    type[j]="upstream" 
                    dist[j]=TS-E
                } else{
                    type[j]="downstream"
                    dist[j]=TE-E
                }
            }
            if(S > TE){
                if(Strand==-1){
                    type[j]="upstream"
                    dist[j]=S-TE
                }  else{
                    type[j]="downstream"
                    dist[j]=S-TS
                }
            }
            ## totally within gene
            if (S >= TS & E <= TE){
                type[j]="inside"
                if(Strand==-1) dist[j]=TE-E  else dist[j]=S-TS
            }
            ## overlaps exactly one side of gene ("covers" done above)
            if(type[j]==""){
                if(S < TS & E <= TE){
                    ##OVERLAP FRONT
                    if(Strand==1) type[j]="overlaps 5'" else{
                        type[j]="overlaps 3'"
                        dist[j]=TE-E
                    }
                }
                else if (S >= TS & E > TE){
                    ##OVERLAP BACK
                    if(Strand==-1) type[j]="overlaps 5'" else{
                        type[j]="overlaps 3'"
                        dist[j]=S-TS
                    }
                }
            }
        }
        
        m1=NA;m2=NA
        if( type[j]%in%c("overlaps 5'","overlaps 3'","inside") & !skipExons ){
            
            ir=IRanges(start=c(S,E),width=1)
            map2 = smarterNearest(ir, exons)
            dist2<-distance(ir,exons[map2])
            pos <- start(ir) ##start end the same
            dist2 <- dist2*ifelse(
                (Strand==1 & 
                 end(exons[map2,]) < pos ) |
                (Strand==-1 &
                 start(exons[map2,]) > pos),-1,1)
            
            tmp<-cbind(dist2,map2)
            m1 = tmp[1,1]
            m2 = tmp[2,1]
            exon1 = tmp[1,2]
            exon2 = tmp[2,2]
            m1m2Index=which.min(abs(c(m1,m2)))
            
            if(exon1==exon2 & m1==0 & m2==0){
                subtype[j]="inside exon"
                insidedistance[j]=0
                exonnumber[j]=c(exon1,exon2)[m1m2Index]
            }
            
            if( (sign(m1)==sign(m2) & (m1!=0 & m2!=0) & exon1==exon2) |
               (sign(m1)==-1 & sign(m2)==1 & exon2-exon1==1) ){
                subtype[j]="inside intron"
                
                insidedistance[j]=c(m1,m2)[m1m2Index]
                exonnumber[j]=c(exon1,exon2)[m1m2Index]
                
            }
            if( (exon2-exon1 > 1) |
               ( (exon2-exon1 == 1) & 
                ((sign(m1)==-1 & sign(m2)==-1) |
                 (sign(m1)==1 & sign(m2)==-1) |
                 (sign(m1)==1 & sign(m2)==1) |
                 (sign(m1)==0 & sign(m2)==-1)|
                 (sign(m1)==1 & sign(m2)==0))) |
               (exon2==exon1 & sign(m1)!=sign(m2))){
                subtype[j]="covers exon(s)"
                insidedistance[j]=0
                exonnumber[j]=c(exon1,exon2)[m1m2Index]
            }
            
            if( (exon2-exon1 <= 1 & sign(m1)==-1 & sign(m2)==0) |
               (exon2==exon1 & sign(m1)==1 & sign(m2)==0)){
                if(Strand==1) subtype[j]="overlaps exon upstream" else subtype[j]="overlaps exon downstream"
                insidedistance[j]=0
                exonnumber[j]=c(exon1,exon2)[m1m2Index]
            }
            if( (exon2-exon1 <= 1 & sign(m1)==0 & sign(m2)==1) |
               (exon2==exon1 & sign(m1)==0 & sign(m2)==-1)){
                if(Strand==-1) subtype[j]="overlaps exon upstream" else subtype[j]="overlaps exon downstream"
                insidedistance[j]=0
                exonnumber[j]=c(exon1,exon2)[m1m2Index]
            }
            if( exon2-exon1 == 1 & sign(m1)==0 & sign(m2)==0){
                subtype[j]="overlaps two exons"
                insidedistance[j]=0
                exonnumber[j]=c(exon1,exon2)[m1m2Index]
            }
            
            if(Strand!=1){
                insidedistance[j]= -insidedistance[j]
                exonnumber[j] = nrow(Exons) - exonnumber[j] + 1
            }
            
            ctype[j]="inside transcription region"
            
            if(!is.na(CS)) {
                if(S<CS & E<CS){
                    if(Strand==1) ctype[j]="5' UTR" else ctype[j]="3'UTR"
                }
            }
            
            
            if(!is.na(CE)) {
                if(S>CE & E>CE){
                    if(Strand==-1) ctype[j]="5' UTR" else ctype[j]="3'UTR"
                }
            }
            
            if(!is.na(CS) & !is.na(CE)) {
                if(S<CS & E>CE){
                    ctype[j]="covers coding region"
                }
            }
            
            if(!is.na(CS)) {
                if(S<CS & E>CS){
                    if(Strand==1) ctype[j]="overlaps 5' UTR" else ctype[j]="overlaps 3'UTR"
                }
            }
            
            if(!is.na(CE)) {
                if(S<CE & E>CE){
                    if(Strand==-1) ctype[j]="overlaps 5' UTR" else ctype[j]="overlaps 3'UTR"
                }
            }
            
            
        }
    ##    if(TE-TS<10^5){##graphical check
            
    ##         plot(0,0,ylim=c(0,0.6),xlim=range(c(start(subject[i,]),end(subject[i,]),start(x[j,]),end(x[j,]))),
    ##              xlab=paste("inside distance=",dist[j],insidedistance[j],m1,m2,exonnumber[j]))
    ##         polygon(c(TS,TE,TE,TS),c(0,0,0.5,0.5),density=0,col=2)
    ##         polygon(c(CS,CE,CE,CS),c(0.1,0.1,0.4,0.4),density=0,col=3)
    ##         abline(h=0.25,lwd=2)
    ##         apply(Exons,1,function(x) polygon(c(x[1],x[2],x[2],x[1]),
    ##                                           c(0.2,0.2,0.3,0.3),col=4))
            
    ##         polygon(c(start(x[j,]),end(x[j,]),end(x[j,]),start(x[j,])),c(0.4,0.4,0.5,0.5),col=5)
    ##         lines(c(TS,TS+1000),c(0.55,0.55),lwd=3)
    ##         title(paste(j,i,Strand,type[j],subtype[j],ctype[j],dist[j],sep=":"))
    ##     }

    }
    
    type[dist<=promoterDist & type=="upstream"] <- "promoter"
    type[dist<=promoterDist & type=="downstream"] <- "close to 3'"
    
    description=type
    tmpIndex=which(description=="inside")
    description[tmpIndex] <- subtype[tmpIndex]
    tmp <- data.frame(name=I(genenames),
                      annotation=I(geneannotation),
                      description=factor(description,levels=c("upstream","promoter","overlaps 5'","inside intron","inside exon","covers exon(s)","overlaps exon upstream","overlaps exon downstream","overlaps two exons","overlaps 3'","close to 3'","downstream","covers")),
                      region=factor(type,levels=c("upstream","promoter","overlaps 5'","inside","overlaps 3'","close to 3'","downstream","covers")),
                      distance=dist,
                      subregion=factor(subtype,levels=c("inside intron","inside exon","covers exon(s)","overlaps exon upstream","overlaps exon downstream","overlaps two exons")),
                      insideDistance=insidedistance,
                      exonnumber=exonnumber,
                      nexons=nexons,
                      UTR=factor(ctype,levels=c("inside transcription region","5' UTR","overlaps 5' UTR","3'UTR","overlaps 3'UTR","covers transcription region")),
                      strand=strands,
                      geneL=geneL,
                      codingL=codingL,
                      Geneid=geneid,
                      subjectHits=map)
    return(tmp)
}                               

poverlapWidth <- function(query, subject)  {
  overlap_score <- pmin(end(query), end(subject)) - 
    pmax(start(query), start(subject)) + 1L
  
  pmax(overlap_score, 0L)
}

smarterNearest <- function(x, subject)  {
  hits <- nearest(x, subject, select="all")
  mcols(hits)$ovwidth <- poverlapWidth(x[queryHits(hits)],subject[subjectHits(hits)])
  
  partitioning <- PartitioningByEnd(queryHits(hits),NG=queryLength(hits))
  ovwidth <- relist(mcols(hits)$ovwidth, partitioning)
  hits <- hits[unlist(ovwidth==max(ovwidth))]
  selectHits(hits, select="arbitrary")
}

context('Annotation-related functions')


## Create toy data with all cases
x <- data.frame(start = c(2, 4, 7, 10, 13, 17, 19, 22, 24), end = c(3, 5, 9, 12, 16, 18, 20, 23, 26), chr = 'chr1')
subject <- data.frame(start = c(1, 4, 7, 11, 14, 21, 25), end = c(3, 6, 9, 12, 15, 22, 27), chr = 'chr1')

## Test cases and types of input
test_that('Annotate Nearest', {
    expect_that(annotateNearest(x, subject, select = 'all'), throws_error())
    expect_that(annotateNearest(x, subject), equals(annotateNearest(x, makeGRangesFromDataFrame(subject))))
    expect_that(annotateNearest(x, subject), equals(annotateNearest(makeGRangesFromDataFrame(x), subject)))
    expect_that(annotateNearest(x, subject)$distance, equals(rep(c(0, -2, 1, 0), c(5, 1, 1, 2))))
    expect_that(annotateNearest(x, subject)$subjectHits, equals(c(1:5, 5:6, 6:7)))
    expect_that(annotateNearest(x, subject)$type, equals(rep(c('inside', 'cover', 'disjoint', 'overlap'), c(2, 3, 2, 2))))
    expect_that(annotateNearest(x, subject)$amountOverlap, equals(rep(c(NA, -1, 2), c(7, 1, 1))))
    expect_that(annotateNearest(x, subject)$insideDist, equals(rep(c(0, NA), c(2, 7))))
    expect_that(annotateNearest(x, subject)$size1, equals(rep(c(2, 3, 4, 2, 3), c(2, 2, 1, 3, 1))))
    expect_that(annotateNearest(x, subject)$size2, equals(rep(c(3, 2, 3), c(3, 5, 1))))
})




## Human (UCSC hg19 knownGene) case
# library('TxDb.Hsapiens.UCSC.hg19.knownGene')
# genes <- annotateTranscripts(TxDb.Hsapiens.UCSC.hg19.knownGene)

## Canis Familiaris case
# library('txdbmaker')
# system.time(can_txdb <- makeTxDbFromUCSC("canFam3", "refGene"))
#    user  system elapsed 
#  70.588   1.002 123.785 

## Using a smaller set is not significantly faster
# system.time(can_txdb <- makeTxDbFromUCSC('canFam3', 'refGene', transcript_ids = c('NM_001193298', 'NM_001002949', 'NM_001080724', 'NM_001003268', 'NM_001252259', 'NM_001012395', 'NM_001286958', 'NM_001284493', 'NM_001122602', 'NM_001003021')))
#   user  system elapsed 
# 61.193   1.035 117.035 

# can_genes <- annotateTranscripts(can_txdb)
# can_genes_NA <- annotateTranscripts(can_txdb, NA)

# test_that('Annotate Transcripts', {
#     expect_that(attributes(can_genes)$description, equals('annotatedTranscripts'))
#     expect_that(attributes(can_genes_NA)$description, equals('annotatedTranscripts'))
#     expect_that(colnames(mcols(can_genes)), equals(colnames((mcols(can_genes_NA)))))
#     expect_that(ranges(can_genes), equals(ranges(can_genes_NA)))
#     expect_that(can_genes_NA$Gene, equals(Rle(NA, length(can_genes_NA))))
#     expect_that(can_genes_NA$Refseq, equals(Rle(NA, length(can_genes_NA))))
#     expect_that(annotateTranscripts(can_txdb, by = 'gene', annotationPackage = 'DoesNotExist', requireAnnotation = TRUE), throws_error())
#     expect_that(sum(can_genes$CSE < can_genes$CSS, na.rm = TRUE), equals(0))
# })



## Test matchGenes()

# test <- head(can_genes)
# strand(test) <- ifelse(strand(test) == "+", "-", "+")
# test2 <- head(can_genes)
# strand(test2) <- '*'
# test3 <- resize(head(can_genes), 10, fix = 'center')
# matchGenes(test, can_genes)


# matched <- matchGenes(head(can_genes), can_genes)
# matched_rev <- matchGenes(test, can_genes)
# test_that('Match genes', {
#     expect_that(matched, equals(matchGenes(test2, can_genes)))
#     expect_that(matchGenes(test3, can_genes)$distance, equals(matched$distance + floor((width(test) - 10) / 2)))
#     expect_that(matchGenes(head(can_genes), can_genes, type = 'fiveprime')$subjectHits, equals(1:6))
#     expect_that(matchGenes(head(can_genes_NA), can_genes_NA)[, 3:ncol(matched)], equals(matched[, 3:ncol(matched)]))
#     expect_that(is.na(matched_rev$codingL[which(matched_rev$Geneid == 104797479)]), equals(TRUE))
#     expect_that(sum(matched_rev$description %in% c('downstream', 'upstream')), equals(6))
#     expect_that(matchGenes(data.frame(start = start(head(can_genes)), end = end(head(can_genes)), chr = seqnames(head(can_genes)), strand = strand(head(can_genes))), can_genes), equals(matched))
# })


## Testing https://github.com/ririzarr/bumphunter/issues/4
library('GenomicRanges')
library('TxDb.Hsapiens.UCSC.hg19.knownGene')
txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
query <- GRanges('chr21', IRanges(22115534, 22115894), strand = "*")
genes <- annotateTranscripts(txdb = txdb)
res <- matchGenes(x = query, subject = genes)

test_that('NA bug', {
    expect_that(as.character(res$UTR), equals('inside transcription region'))
    expect_that(res$insideDistance, equals(0))
    expect_that(as.character(res$description), equals('inside exon'))
})


## Test using a GENCODE v36 txdb object
## Details from https://github.com/rafalab/bumphunter/issues/15

library('rtracklayer')
## Get the raw data
gr <- import('http://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_human/release_36/gencode.v36.annotation.gtf.gz')

## Subset and add the chromosome length info
gr_small <- GenomeInfoDb::keepSeqlevels(gr, c('chrY', 'chrM'), pruning.mode = 'tidy')
hg38_chrominfo <- GenomeInfoDb::getChromInfoFromUCSC("hg38")
new_info <- hg38_chrominfo$UCSC_seqlength[match(seqlevels(gr_small),
    hg38_chrominfo$UCSC_seqlevel)]

## Create a small TxDb object
txdb <- txdbmaker::makeTxDbFromGRanges(gr_small)

ann <- annotateTranscripts(txdb, annotationPackage = 'org.Hs.eg.db',
    mappingInfo = list('column' = 'ENTREZID', 'keytype' = 'ENSEMBL',
    'multiVals' = 'first'), simplifyGeneID = TRUE)

## Annotate some dummy regions
genes_gencode <- matchGenes(ann[which(ann$Gene == 'CD99')], ann)

## Check the reuslt
test_that('Gencode v36 genes', {
    expect_equal(unique(gr_small[grepl(genes_gencode$Geneid[1], gr_small$gene_id)]$gene_name), unique(genes_gencode$name))
    expect_equal('CD99', unique(genes_gencode$name))
})

greaterOrEqual <- function(x,y) {
    precision <- sqrt(.Machine$double.eps)
    (x >= y) | (abs(x-y) <= precision)
}

closeSockets <- function() {
    allCon <- showConnections()
    socketCon <- as.integer(rownames(allCon)[allCon[, "class"] == "sockconn"])
    sapply(socketCon, function(ii) close.connection(getConnection(ii)) )
}

foreachCleanup <- function() {
    if (exists(".revoDoParCluster", where=doParallel:::.options)) {
        if(!is.null(doParallel:::.options$.revoDoParCluster))
            stopCluster(doParallel:::.options$.revoDoParCluster)
        remove(".revoDoParCluster", envir=doParallel:::.options)
    }
}

.getSegments_old <- function(x, factor, cutoff=quantile(abs(x),0.99), verbose=TRUE){
    Indexes <- split(seq(along=x),factor)
    regionID <- vector("numeric", length(x))
    LAST <- 0
  
    segmentation <- vector("numeric", length(x))
    type <- vector("numeric", length(x))

    for (i in seq(along = Indexes)) {
        if (verbose) if (i%%1000 == 0) cat(".")
        Index <- Indexes[[i]]
        y <- x[Index]
        z <- sign(y) * as.numeric(abs(y) > cutoff)
        w <- cumsum(c(1, diff(z) != 0)) + LAST
        segmentation[Index] <- w
        type[Index] <- z
        LAST <- max(w)
    }
    ##add a vector of the pns
    res <- list(upIndex = split(which(type>0), segmentation[type>0]),
                dnIndex = split(which(type<0), segmentation[type<0]),
                zeroIndex = split(which(type==0), segmentation[type==0]))
    names(res[[1]]) <- NULL
    names(res[[2]]) <- NULL
    names(res[[3]]) <- NULL
    return(res)
}

getSegments <- function(x, f = NULL, cutoff=quantile(abs(x), 0.99), assumeSorted = FALSE, verbose=FALSE){
    if(is.null(f))
        f <- rep(1L, length(x))
    stopifnot(length(x) == length(f))
    stopifnot(length(cutoff) <= 2)
    if(is.character(f))
        f <- as.factor(f)
    if(is.numeric(f))
        f <- as.integer(f)
    stopifnot(is.factor(f) || is.integer(f))
    if(length(cutoff) == 1)
        cutoff <- c(-cutoff, cutoff)
    cutoff <- sort(cutoff)

    reordered <- FALSE
    if(!assumeSorted && is.unsorted(f)) {
	od <- order(f)
	x <- x[od]
	f <- f[od]
	reordered <- TRUE
    }
        
    if(verbose) message("getSegments: segmenting")
    Indexes <- split(seq(along=x), f)
    direction <- as.integer(greaterOrEqual(x, cutoff[2]))
    direction[x <= cutoff[1]] <- -1L

    ## We now need to convert f into cid
    if(verbose) message("getSegments: splitting")
    segments <- cumsum(c(1, diff(direction) != 0)) +
        cumsum(c(1, diff(f) != 0))
    names(segments) <- NULL

    res <- list(upIndex = split(which(direction>0), segments[direction>0]),
                dnIndex = split(which(direction<0), segments[direction<0]),
                zeroIndex = split(which(direction==0), segments[direction==0]))
    names(res[[1]]) <- NULL
    names(res[[2]]) <- NULL
    names(res[[3]]) <- NULL

    if(reordered) {
        res <- lapply(res, function(sp) lapply(sp, function(xx) od[xx]))
    }
    res
}

clusterMaker <- function(chr, pos, assumeSorted = FALSE, maxGap=300){

    if( (any(is.na(chr))) || (any(is.na(pos))) ) stop("NAs found in chr and/or pos vector(s). Please replace NA values in chr and/or pos vector(s) to actual chromosome names and genomic locations. ")
    
    nonaIndex <- which(!is.na(chr) & !is.na(pos))
    Indexes <- split(nonaIndex, chr[nonaIndex])
    clusterIDs <- rep(NA, length(chr))
    LAST <- 0
    for(i in seq(along = Indexes)){
        Index <- Indexes[[i]]
        x <- pos[Index]
        if(!assumeSorted){
            Index <- Index[order(x)]
            x <- pos[Index]
        }
        y <- as.numeric(diff(x) > maxGap)
        z <- cumsum(c(1, y))
        clusterIDs[Index] <- z + LAST
        LAST <- max(z) + LAST
    }
    clusterIDs
}

##you can pass cutoff through the ...
regionFinder <- function(x, chr, pos, cluster=NULL, y=x, summary=mean,
                         ind=seq(along=x),order=TRUE, oneTable=TRUE,
                         maxGap=300, cutoff=quantile(abs(x), 0.99),
                         assumeSorted = FALSE, verbose = TRUE){
    if(any(is.na(x[ind]))){
        warning("NAs found and removed. ind changed.")
        ind <- intersect(which(!is.na(x)),ind)
    } 
    if(is.null(cluster))
        cluster <- clusterMaker(chr, pos, maxGap=maxGap, assumeSorted = assumeSorted)
    Indexes <- getSegments(x = x[ind], f = cluster[ind], cutoff = cutoff,
                           assumeSorted = assumeSorted, verbose = verbose)
    clusterN <- table(cluster)[as.character(cluster)]
    
    res <- vector("list",2)
    for(i in 1:2){
      res[[i]]<-
        data.frame(chr=sapply(Indexes[[i]],function(Index) chr[ind[Index[1]]]),
                   start=sapply(Indexes[[i]],function(Index) min(pos[ind[Index]])),
                   end=sapply(Indexes[[i]], function(Index) max(pos[ind[Index]])),
                   value=sapply(Indexes[[i]],function(Index)summary(y[ind[Index]])),
                   area=sapply(Indexes[[i]],function(Index)abs(sum(y[ind[Index]]))),
                   cluster=sapply(Indexes[[i]],function(Index)cluster[ind[Index]][1]),
                   indexStart=sapply(Indexes[[i]], function(Index) min(ind[Index])),
                   indexEnd = sapply(Indexes[[i]], function(Index) max(ind[Index])),
                   stringsAsFactors=FALSE)

      res[[i]]$L <- res[[i]]$indexEnd - res[[i]]$indexStart+1
      res[[i]]$clusterL <- sapply(Indexes[[i]], function(Index) clusterN[ind[Index]][1])
    }
    names(res) <- c("up","dn")
    if(order & !oneTable){
        if(nrow(res$up)>0) res$up <- res$up[order(-res$up$area),]
        if(nrow(res$dn)>0) res$dn <- res$dn[order(-res$dn$area),]
    }
    if(oneTable){
        res <- rbind(res$up,res$dn)
        if(order & nrow(res)>0) res <- res[order(-res$area),]
    }
    return(res)
}

boundedClusterMaker <- function(chr, pos, assumeSorted = FALSE,
                                maxClusterWidth = 1500, maxGap = 500) {

    if( (any(is.na(chr))) || (any(is.na(pos))) ) stop("NAs found in chr and/or pos vector(s). Please replace NA values in chr and/or pos vector(s) to actual chromosome names and genomic locations. ")

    nonaIndex <- which(!is.na(chr) & !is.na(pos))
    Indexes <- split(nonaIndex, chr[nonaIndex])
    clusterIDs <- rep(NA, length(chr))
    LAST <- 0
    for (i in seq(along = Indexes)) {
        Index <- Indexes[[i]]
        x <- pos[Index]
        if (!assumeSorted) {
            Index <- Index[order(x)]
            x <- pos[Index]
        }
        y <- as.numeric(diff(x) > maxGap)
        z <- cumsum(c(1, y))
        startPosition <- tapply(x, list(z), min)
        startPositions <- startPosition[as.character(z)]

        offset <- x - startPositions
        addToZ <- offset %/% maxClusterWidth
        addToZ <- cumsum(addToZ * c(0, as.numeric(diff(addToZ) > 0)))
        z <- z + addToZ
        clusterIDs[Index] <- z + LAST
        LAST <- max(z) + LAST
    }
    clusterIDs
}

smoother <- function(y, x=NULL, cluster, weights=NULL,
                     smoothFunction, verbose=TRUE, ...) {
    if(is.null(dim(y)))
        y <- matrix(y, ncol=1) ##need to change this to be more robust
    if(!is.null(weights) && is.null(dim(weights)))
        weights <- matrix(weights, ncol=1)
    if (!getDoParRegistered())
        registerDoSEQ()

    cores <- getDoParWorkers()
    Indexes <- split(seq(along=cluster), cluster)
    baseSize <- length(Indexes) %/% cores
    remain <- length(Indexes) %% cores
    done <- 0L
    IndexesChunks <- vector("list", length = cores)
    for(ii in 1:cores) {
        if(remain > 0) {
            IndexesChunks[[ii]] <- done + 1:(baseSize + 1)
            remain <- remain - 1L
            done <- done + baseSize + 1L
        } else {
            IndexesChunks[[ii]] <- done + 1:baseSize
            done <- done + baseSize
        }
    }
    IndexesChunks <- lapply(IndexesChunks, function(idxes) {
        do.call(c, unname(Indexes[idxes]))
    })
    idx <- NULL ## for R CMD check
    ret <- foreach(idx = iter(IndexesChunks), .packages = "bumphunter") %dorng% {
        sm <- smoothFunction(y=y[idx,], x=x[idx], cluster=cluster[idx],
                             weights=weights[idx,], verbose=verbose, ...)
        c(sm, list(idx = idx))
    }
    
    attributes(ret)[["rng"]] <- NULL
    ## Paste together results from different workers
    ret <- reduceIt(ret)

    ## Now fixing order issue
    revOrder <- ret$idx
    names(revOrder) <- seq_along(ret$idx)
    revOrder <- sort(revOrder)
    revOrder <- as.integer(names(revOrder))
    
    ret$smoother <- ret$smoother[1]
    ret$fitted <- ret$fitted[revOrder,,drop=FALSE]
    ret$smoothed <- ret$smoothed[revOrder]
    ret$idx <- NULL
    return(ret)  
}

locfitByCluster <- function(y, x = NULL, cluster, weights = NULL,
                            minNum = 7, bpSpan = 1000, minInSpan = 0,
                            verbose = TRUE) {
    ## Depends on limma
    ## bpSpan is in basepairs
    ## assumed x are ordered
    ## if y is vector change to matrix
    if(is.null(dim(y)))
        y <- matrix(y, ncol = 1) 
    if(!is.null(weights) && is.null(dim(weights)))
        weights <- matrix(weights, ncol = 1)
    if(is.null(x))
        x <- seq(along = y)
    if(is.null(weights))
        weights <- matrix(1, nrow = nrow(y), ncol = ncol(y))
    Indexes <- split(seq(along = cluster), cluster)
    clusterL <- sapply(Indexes, length)
    smoothed <- rep(TRUE, nrow(y))
    
    for(i in seq(along=Indexes)) {
        if(verbose) if(i %% 1e4 == 0) cat(".")
        Index <- Indexes[[i]]
        if(clusterL[i] >= minNum & sum(rowSums(is.na(y[Index,,drop=FALSE]))==0) >= minNum)  {
            nn <- minInSpan / length(Index)
            for(j in 1:ncol(y)) {
                sdata <- data.frame(pos = x[Index],
                                    y = y[Index, j],
                                    weights = weights[Index,j])
                fit <- locfit(y ~ lp(pos, nn = nn, h = bpSpan), data = sdata,
                              weights = weights, family = "gaussian", maxk = 10000)
                pp <- preplot(fit, where = "data", band = "local",
                              newdata = data.frame(pos = x[Index]))
                y[Index,j] <- pp$trans(pp$fit)
            }
        } else {
            y[Index,] <- NA
            smoothed[Index] <- FALSE
        }
    }                
    return(list(fitted=y, smoothed=smoothed, smoother="locfit"))
}

loessByCluster <- function(y, x=NULL, cluster, weights= NULL,
                         bpSpan = 1000, minNum=7, minInSpan=5,
                         maxSpan=1, verbose=TRUE) {
    ## Depends on limma
    ## bpSpan is in basepairs
    ## assumed x are ordered
    ## if y is vector change to matrix
    if(is.null(dim(y)))
        y <- matrix(y, ncol=1) ##need to change this to be more robust
    if(!is.null(weights) && is.null(dim(weights)))
        weights <- matrix(weights,ncol=1)
    
    if(is.null(x))
        x <- seq(along=y)
    if(is.null(weights))
        weights <- matrix(1, nrow=nrow(y), ncol=ncol(y))
    Indexes <- split(seq(along=cluster), cluster)
    clusterL <- sapply(Indexes, length)
    spans <- rep(NA, length(Indexes))
    smoothed <- rep(TRUE, nrow(y))
    
    for(i in seq(along=Indexes)) {
        if(verbose) if(i %% 1e4 == 0) cat(".")
        Index <- Indexes[[i]]
        if(clusterL[i] >= minNum) {
            span = bpSpan/median(diff(x[Index]))/length(Index) # make a span
            if(span > maxSpan) span <- maxSpan
            spans[i] <- span
            if(span*length(Index)>minInSpan){
                ##this can be parallelized
                for(j in 1:ncol(y)){
                    y[Index,j] <- limma::loessFit(y[Index,j], x[Index], span = span,
                              weights = weights[Index,j])$fitted
                }
            } else{
                y[Index,] <- NA
                smoothed[Index] <- FALSE
            }
        } else{
            y[Index,] <- NA
            spans[i] <- NA
            smoothed[Index] <- FALSE
        }
    }
    
    return(list(fitted=y, smoothed=smoothed, smoother="loess"))
}


runmedByCluster <- function(y, x=NULL, cluster, weights=NULL,
                            k=5, endrule="constant", verbose=TRUE) {
    ## we dont use chr and pos byt
    ## Depends on limma
    ## bpSpan is in basepairs
    ## assumed y are ordered
    ## x is ingored (for now... should be used for ties)
    ## weight is ingored too. x and weight are included for consistency
    ## in parameters
    if(is.null(dim(y)))
        y <- matrix(y, ncol=1) ##need to change this to be
    
    Indexes <- split(seq(along=cluster), cluster)
    clusterL <- sapply(Indexes, length)
    spans <- rep(k, length(Indexes)) ##using spans for consistency with loessByClusters
    smoothed <- rep(TRUE, nrow(y))
    ## We can parallelize here
    for(i in seq(along=Indexes)) {
        if(verbose) if(i %% 1e4 == 0) cat(".")
        Index <- Indexes[[i]]
        if(clusterL[i] >= k) {
            for(j in 1:ncol(y)){
                y[Index,j] <- runmed(y[Index,j], k=k, endrule=endrule)
            }
        } else {
            y[Index,] <- NA
            smoothed[Index] <- FALSE
        }
    }
    return(list(fitted=y, smoothed=smoothed, smoother="runmed"))
}


reduceIt <- function(x, elem, bind=rbind) {
    if(missing(elem)) elem <- names(x[[1]])
    ret <- lapply(elem, function(el) {
        xx <- lapply(x, "[[", el)
        if (is.matrix(xx[[1]])) return(do.call(bind, xx))
        else if (is.vector(xx[[1]])) return(do.call("c", xx))
        else stop("reduce can only handle matrices or vectors")
    })
    names(ret) <- elem
    ret
}

test_loessByCluster <- function () {
    set.seed(123)
    dat <- dummyData()
    smoothed <- loessByCluster(y=dat$mat[,1], cluster=dat$cluster, bpSpan = 1000,
                               minNum=7, minInSpan=5, maxSpan=1)
    load(file.path(path.package("bumphunter"), "unitTests/loessSmoothed.rda"))
    checkEquals(smoothed, loessSmoothed)
}

test_runmedByCluster <- function () {
    set.seed(123)
    dat <- dummyData()
    smoothed <- runmedByCluster(y=dat$mat[,1], cluster=dat$cluster, k=5, endrule="constant")
    load(file.path(path.package("bumphunter"), "unitTests/runmedSmoothed.rda"))
    checkEquals(smoothed, runmedSmoothed)
}

test_smoother <- function() {
    set.seed(123)
    dat <- dummyData()
    ## loessByCluster
    smoothed <- smoother(y=dat$mat[,1], cluster=dat$cluster,
                         smoothFunction=loessByCluster, 
                         bpSpan = 1000, minNum=7, minInSpan=5, maxSpan=1)
    load(file.path(path.package("bumphunter"), "unitTests/loessSmoothed.rda"))
    checkEquals(smoothed, loessSmoothed)
    ## runmedByCluster
    smoothed <- smoother(y=dat$mat[,1], cluster=dat$cluster,
                         smoothFunction=runmedByCluster, 
                         k=5, endrule="constant")
    load(file.path(path.package("bumphunter"), "unitTests/runmedSmoothed.rda"))
    checkEquals(smoothed, runmedSmoothed)  
}

setMethod("bumphunter", signature(object = "matrix"),
          function(object, design, chr=NULL, pos, cluster=NULL,
                   coef=2, cutoff=NULL, pickCutoff=FALSE, pickCutoffQ=0.99,
                   maxGap=500,
                   nullMethod=c("permutation","bootstrap"), smooth=FALSE,
                   smoothFunction=locfitByCluster,
                   useWeights=FALSE, B=ncol(permutations), permutations=NULL,
                   verbose=TRUE, ...){
              nullMethod  <- match.arg(nullMethod)
              if(missing(design)) stop("design must be specified")
              if(missing(pos)) stop("If object is a matrix, pos must be specified")
              bumphunterEngine(object, design=design, chr=chr, pos,
                               cluster=cluster,
                               coef=coef,
                               cutoff=cutoff, pickCutoff=pickCutoff,
                               pickCutoffQ=pickCutoffQ,
                               maxGap=maxGap,nullMethod=nullMethod,
                               smooth=smooth,
                               smoothFunction=smoothFunction,
                               useWeights=useWeights, B=B,
                               permutations=NULL,
                               verbose=verbose, ...)
          })


bumphunterEngine<-function(mat, design, chr = NULL, pos,
                           cluster = NULL, coef = 2,
                           cutoff = NULL, pickCutoff = FALSE,
                           pickCutoffQ = 0.99,
                           maxGap = 500,
                           nullMethod = c("permutation","bootstrap"),
                           smooth = FALSE,
                           smoothFunction = locfitByCluster,
                           useWeights = FALSE,
                           B = ncol(permutations),
                           permutations = NULL,
                           verbose = TRUE, ...){
    nullMethod  <- match.arg(nullMethod)
    if (is.null(B))  B = 0
    if (!is.matrix(permutations) & !is.null(permutations)) stop("permutations must be NULL or a matrix.")
    if (!is.null(permutations)) {
        if (nrow(design) != nrow(permutations))
            stop("Number of rows of 'design' must match number of rows of 'permutations'.")
        if (B != ncol(permutations)) {
            warning("Ignoring the supplied B. Using 'ncol(permutations)' instead.")
            B = ncol(permutations)
        }
    }
     if (ncol(design) > 2 & B > 0 & nullMethod == "permutation"){
        message("[bumphunterEngine] The use of the permutation test is not recommended with multiple covariates, (ncol(design)>2). Consider changing 'nullMethod' changed to 'bootstrap' instead. See vignette for more information.")
        warning("The use of the permutation test is not recommended with multiple covariates, (ncol(design)>2). Consider changing 'nullMethod' changed to 'bootstrap' instead. See vignette for more information.")
    }
    if (!is.matrix(mat))
        stop("'mat' must be a matrix.")
    if (ncol(mat) != nrow(design))
        stop("Number of columns of 'mat' must  match number of rows of 'design'")
    if (B < 0)
        stop("'B' has to be an integer greater or equal to zero")
    if (!(is.null(cutoff) || length(cutoff) %in% 1:2))
        stop("'cutoff' has to be either NULL or a vector of length 1 or 2")
    if (length(cutoff) == 2)
        cutoff <- sort(cutoff)
    if (is.null(cutoff) && !pickCutoff)
        stop("Must pick a cutoff, either using 'cutoff' or 'pickCutoff'")
    if (!is.null(cutoff) && pickCutoff) {
        pickCutoff <- FALSE
        warning("'cutoff' was supplied so ignoring 'pickCutoff=TRUE'")
    }
    if (pickCutoff && (length(pickCutoffQ) != 1 || pickCutoff <  0 || pickCutoffQ > 1))
        stop("Using `pickCutoff = TRUE' requires that 'pickCutoffQ' is a single number between 0 and 1")
    if (pickCutoff && B < 1)
        stop("Must do at least one permution to pick a cutoff")
    if (!getDoParRegistered())
        registerDoSEQ()
    workers <- getDoParWorkers()
    backend <- getDoParName()
    version <- getDoParVersion()
    subverbose <- max(as.integer(verbose) - 1L, 0)
    if (verbose) {
        if (workers == 1) {
            mes <- "[bumphunterEngine] Using a single core (backend: %s, version: %s)."
            message(sprintf(mes, backend, version))
        }
        else {
            mes <- "[bumphunterEngine] Parallelizing using %s workers/cores (backend: %s, version: %s)."
            message(sprintf(mes, workers, backend, version))
        }
    }
    if (is.null(chr))
        chr <- rep("Unspecified", length(pos))
    if (is.factor(chr))
        chr <- as.character(chr)
    if (is.null(cluster))
        cluster <- clusterMaker(chr, pos, maxGap = maxGap)
    if (verbose)
        message("[bumphunterEngine] Computing coefficients.")
    if (useWeights & smooth) { 
        tmp <- .getEstimate(mat = mat, design = design,
                            coef = coef, full = TRUE)
        rawBeta <- tmp$coef
        weights <- 1/tmp$sigma
        rm(tmp)
    } else {
        rawBeta <- .getEstimate(mat = mat, design = design, coef = coef,
            full = FALSE)
        weights <- NULL
    }
    if (smooth) {
        if (verbose)
            message("[bumphunterEngine] Smoothing coefficients.")
        beta <- smoother(y = rawBeta, x = pos, cluster = cluster,
            weights = weights, smoothFunction = smoothFunction,
            verbose = subverbose, ...)
        Index <- which(beta$smoothed)
        beta <- beta$fitted
    } else {
        beta <- rawBeta
        Index <- seq(along = beta)
    }
    if (B > 0) {
        if (nullMethod == "permutation"){
            if (verbose)
                message("[bumphunterEngine] Performing ", B, " permutations.")
            if (useWeights && smooth) {
                tmp <- .getEstimate(mat, design, coef, B,
                                    permutations, full = TRUE)
                permRawBeta <- tmp$coef
                weights <- tmp$sigma
                rm(tmp)
            }
            else {
                permRawBeta <- .getEstimate(mat, design, coef, B, permutations, full = FALSE)
                weights <- NULL
            }
            NullBeta<-permRawBeta	
        }
        if (nullMethod == "bootstrap"){
            message("[bumphunterEngine] Performing ", B, " bootstraps.")
            qr.X <- qr(design)
            resids <- t(tcrossprod( diag(nrow(design)) - tcrossprod(qr.Q(qr.X)),  mat))

            ##rescale residuals
            h <- diag(tcrossprod(qr.Q( qr(design))))
            resids <- t(t(resids)/sqrt(1-h))

            ##create the null model to which we add bootstrap resids
            design0 <- design[,-coef,drop=FALSE]
            qr.X0 <- qr(design0)
            null <- t(tcrossprod(tcrossprod(qr.Q(qr.X0)), mat))

            ##Now do the boostraps
            chunksize <- ceiling(B/workers) 
            bootIndexes<-replicate(B, sample(1:ncol(mat),replace=TRUE),simplify=TRUE)
            
            tmp <- foreach(bootstraps = iter(bootIndexes, by = "column", chunksize = chunksize),
                           .combine = "cbind", .packages = "bumphunter") %dorng% {
                               apply(bootstraps, 2, function(bootIndex){
                                   ##create a null model
                                   matstar <- null+resids[,bootIndex]
                                   ##compute the null beta estimate
                                   nullbetas <- backsolve(qr.R(qr.X),crossprod(qr.Q(qr.X),t(matstar)))[coef,]
                                   if (useWeights){
                                       ##compute sigma
                                       sigma <- rowSums(t(tcrossprod( diag(nrow(design)) - tcrossprod(qr.Q(qr.X)),  matstar))^2)
                                       sigma <-
                                           sqrt(sigma/(nrow(design)-qr.X$rank))
                                       outList <- list(coef=nullbetas,sigma=sigma)
                                   } else {
                                       outList <- nullbetas
                                   }
                                   return(outList)
                               })
                           }
            
            if (useWeights && smooth) {
                bootRawBeta <- do.call(Map, c(cbind, tmp))$coef
                weights <- do.call(Map, c(cbind, tmp))$sigma
            } else {
                bootRawBeta <- tmp
                weights <- NULL
            }
            NullBeta<-bootRawBeta
            rm(tmp)
            rm(bootRawBeta)
        }	
        if (verbose)
            message("[bumphunterEngine] Computing marginal ",nullMethod," p-values.")
        sumGreaterOrEqual <- rowSums(greaterOrEqual(abs(NullBeta),
            abs(as.vector(rawBeta))))
        pvs <- (sumGreaterOrEqual + 1L)/(B + 1L)
        if (smooth) {
            if (verbose)
                message("[bumphunterEngine] Smoothing ",nullMethod," coefficients.")
            permBeta <- smoother(y = NullBeta, x = pos, cluster = cluster,
                weights = weights, smoothFunction = smoothFunction,
                verbose = subverbose, ...)$fitted
        } else permBeta <- NullBeta
        if (is.null(cutoff))
            cutoff <- quantile(abs(permBeta), pickCutoffQ, na.rm = TRUE)
        if (verbose)
            message(sprintf("[bumphunterEngine] cutoff: %s",
                round(cutoff, 3)))
    }
    if (verbose)
        message("[bumphunterEngine] Finding regions.")
    tab <- regionFinder(x = beta, chr = chr, pos = pos, cluster = cluster,
        cutoff = cutoff, ind = Index, verbose = FALSE)
    if (nrow(tab) == 0) {
        if (verbose)
            message("[bumphunterEngine] No bumps found!")
        return(list(table = NA, coef = rawBeta, fitted = beta,
            pvaluesMarginal = NA))
    } else {
        if (verbose)
            message(sprintf("[bumphunterEngine] Found %s bumps.",
                nrow(tab)))
    }
    if (B < 1) {
        return(list(table = tab, coef = rawBeta, fitted = beta,
            pvaluesMarginal = NA))
    }
    if (verbose)
        message("[bumphunterEngine] Computing regions for each ",nullMethod,".")
    chunksize <- ceiling(B/workers)
    subMat <- NULL
    nulltabs <- foreach(subMat = iter(permBeta, by = "col", chunksize = chunksize),
        .combine = "c", .packages = "bumphunter") %dorng% {
        apply(subMat, 2, regionFinder, chr = chr, pos = pos,
            cluster = cluster, cutoff = cutoff, ind = Index,
            verbose = FALSE)
    }
    attributes(nulltabs)[["rng"]] <- NULL
    if (verbose)
        message("[bumphunterEngine] Estimating p-values and FWER.")
    L <- V <- A <- as.list(rep(0, B))
    for (i in 1:B) {
        nulltab <- nulltabs[[i]]
        if (nrow(nulltab) > 0) {
            L[[i]] <- nulltab$L
            V[[i]] <- nulltab$value
            A[[i]] <- nulltab$area
        }
    }
    computation.tots <- function(tab, V, L) {
        Lvalue <- cbind(tab$L, abs(tab$value))
        chunksize <- ceiling(nrow(Lvalue)/workers)
        subL <- NULL
        tots <- foreach(subL = iter(Lvalue, by = "row", chunksize = chunksize),
                        .combine = "cbind", .packages = "bumphunter") %dorng%
        {
                apply(subL, 1, function(x) {
                    res <- sapply(seq(along = V), function(i) {
                      sum(greaterOrEqual(L[[i]], x[1]) &
                          greaterOrEqual(abs(V[[i]]),
                        x[2]))
                  })
                  c(mean(res > 0), sum(res))
                })
            }
        attributes(tots)[["rng"]] <- NULL
        rate1 <- tots[1, ]
        pvalues1 <- tots[2, ]/sum(sapply(nulltabs, nrow))
        return(list(rate1 = rate1, pvalues1 = pvalues1))
    }
##    ptime1 <- proc.time()
    comp <- computation.tots(tab = tab, V = V, L = L)
    rate1 <- comp$rate1
    pvalues1 <- comp$pvalues1
##    ptime2 <- proc.time()
    computation.tots2 <- function(tab, A) {
        Avalue <- matrix(tab$area, ncol = 1)
        chunksize <- ceiling(nrow(Avalue)/workers)
        subA <- NULL
        tots2 <- t(foreach(subA = iter(Avalue, by = "row", chunksize = chunksize),
            .combine = "cbind", .packages = "bumphunter") %dorng%
            {
                sapply(subA, function(x) {
                  return(sapply(seq(along = A), function(i) {
                    sum(greaterOrEqual(A[[i]], x[1]))
                  }))
                })
            })
        if (is.vector(tots2)) {
            tots2 <- matrix(tots2, nrow = 1)
        }
        rate2 <- rowMeans(tots2 > 0)
        pvalues2 <- rowSums(tots2)/sum(sapply(nulltabs, nrow))
        return(list(rate2 = rate2, pvalues2 = pvalues2))
    }
    ##ptime1 <- proc.time()
    comp <- computation.tots2(tab = tab, A = A)
    rate2 <- comp$rate2
    pvalues2 <- comp$pvalues2
    ##ptime2 <- proc.time()
    tab$p.value <- pvalues1
    tab$fwer <- rate1
    tab$p.valueArea <- pvalues2
    tab$fwerArea <- rate2
    tab <- tab[order(tab$fwer, -tab$area), ]
    algorithm <- list(version = packageDescription("bumphunter")$Version,
        coef = coef, cutoff = cutoff, pickCutoff = pickCutoff,
        pickCutoffQ = pickCutoffQ, smooth = smooth, maxGap = maxGap, nullMethod=nullMethod,
        B = B, permutations = permutations, useWeights = useWeights,
        smoothFunction = deparse(substitute(smoothFunction)))
    ret <- list(table = tab, coef = rawBeta, fitted = beta, pvaluesMarginal = pvs,
        null = list(value = V, length = L), algorithm = algorithm)
    class(ret) <- "bumps"
    return(ret)
}


print.bumps <- function(x, ...) {
    cat(sprintf("a 'bumps' object with %s bumps\n", nrow(x$table)))
}

.getEstimate <- function(mat, design, coef, B=NULL, permutations=NULL,
                         full=FALSE) {
    v <- design[,coef]
    A <- design[,-coef, drop=FALSE]
    qa <- qr(A)
    S <- diag(nrow(A)) - tcrossprod(qr.Q(qa))

    vv <- if (is.null(B)) matrix(v, ncol=1) else{
        if(is.null(permutations)){
            replicate(B, sample(v))
        } else{
            apply(permutations,2,function(i) v[i])
        }
    }
            
            
    sv <- S %*% vv
    vsv <- diag(crossprod(vv,sv))
    
    b <- (mat %*% crossprod(S, vv)) / vsv
    if(!is.matrix(b))
        b <- matrix(b, ncol = 1)
    if (full) {
        sy <- mat %*% S
        df.residual <- ncol(mat) - qa$rank - 1
        if (is.null(B)) {
            sigma <- matrix(sqrt(rowSums((sy - tcrossprod(b, sv))^2) / df.residual), ncol=1)
        } else {
            sigma <- b
            tmp <- sy
            for (j in 1:B) {
                tmp <- tcrossprod(b[,j], sv[,j])
                sigma[,j] <- rowSums((sy-tmp)^2)
            }
            sigma <- sqrt(sigma/df.residual)
        }
        out <- list(coef=b,
                    sigma=sigma,
                    stdev.unscaled=sqrt(1/vsv),
                    df.residual=df.residual)
        if (is.null(B)) out$stdev <- as.numeric(out$stdev)
    } else {
        out <- b
    }
    return(out)
}

.getModT <- function(obj) {
    s2 <- apply(obj$sigma^2, 2, limma::squeezeVar, obj$df.residual)
    out <- obj$coef
    for (j in 1:ncol(out)) {
        out[,j] <- out[,j] / obj$stdev.unscaled[j] / sqrt(s2[[j]]$var.post)
    }
    df.total <- obj$df.residual + sapply(s2,"[[","df.prior")
    return(list(t=out, df.total=df.total))
}

%\VignetteIndexEntry{The bumphunter user's guide}
%\VignetteDepends{bumphunter}
%\VignetteDepends{doParallel}
%\VignettePackage{bumphunter}
\documentclass[12pt]{article}
<<options,echo=FALSE,eval=TRUE,results=hide>>=
options(width=70)
@
\SweaveOpts{eps=FALSE,echo=TRUE}
\usepackage{times}
\usepackage{color,hyperref}
\usepackage{fullpage}
\usepackage[numbers]{natbib}
\definecolor{darkblue}{rgb}{0.0,0.0,0.75}
\hypersetup{colorlinks,breaklinks,
            linkcolor=darkblue,urlcolor=darkblue,
            anchorcolor=darkblue,citecolor=darkblue}

\newcommand{\Rcode}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\texttt{#1}}}
\newcommand{\software}[1]{\textsf{#1}}
\newcommand{\R}{\software{R}}
\newcommand{\bold}[1]{\textbf{#1}} % Fix for R bibentry

\title{The bumphunter user's guide}
\author{Kasper Daniel Hansen \texttt{khansen@jhsph.edu}
  \and
  Martin Aryee \texttt{aryee.martin@mgh.harvard.edu}
  \and
  Rafael A. Irizarry \texttt{rafa@jhu.edu}
}
\date{Modified: November 23, 2012.  Compiled: \today}
\begin{document}
\setlength{\parskip}{1\baselineskip}
\setlength{\parindent}{0pt}

\maketitle

\section*{Introduction}

This package implements the statistical procedure described in \cite{Jaffe:2012} (with some small
modifications).  Notably, batch effect removal 
and the application of the bootstrap to linear models of Efron and
Tibshirani  \cite{bootstrap} need additional code.

For any given type of data, it is usually necessary to make a number of choices and/or
transformations before the bump hunting methodology is ready to be applied.  Typically, these
modifications resides in other packages.  Examples are \Rpackage{charm} (for CHARM-like methylation
microarrays), \Rpackage{bsseq} (for whole-genome bisulfite sequencing data) and \Rpackage{minfi}
(for Illumina 450k methylation arrays).  In some cases (specifically \Rpackage{bsseq}) only parts of
the methodology as implemented in the \Rpackage{bumphunter} package is applied, although the
conceptual approach is still build on bump hunting.

In other words, this package is mostly intended for developers wishing to adapt the general
methodology to their specific applications.

The core of the package is encapsulated in the \Rcode{bumphunter} method which uses the underlying
\Rcode{bumphunterEngine} to do the heavy lifting.  However, \Rcode{bumphunterEngine} consists of a
number of useful functions that does much of the specific tasks involved in bump hunting.  This
document attempts to describe the overall workflow as well as the specific functions.  The relevant
functions are \Rcode{clusterMaker}, \Rcode{getSegments}, \Rcode{findRegions}.


<<libload>>=
library(bumphunter)
@

Note that this package is written with genomic data as an illustrative example but most of it is
easily generalizable to other data types.

\subsection*{Other functions}

Most of the \Rpackage{bumphunter} package is code for bump hunting.  But we also include a number of
convenience functions we have found useful, which are not necessarily part of the bump hunting
exercise.  At the time of writing, this include \Rcode{annotateNearest}.

\section*{The Statistical Model}

The bump hunter methodology is meant to work on data with several biological replicates, similar to
the \Rcode{lmFit} function in \Rpackage{limma}.  While the package is written using genomic data as
an illustrative example, most of it is generalizable to other data types (with some one-dimensional
location information).

We assume we have data $Y_{ij}$ where $i$ represents (biological) replicate and $l_j$ represents
genomic location.  The use of $j$ and $l_j$ is a convenience notation, allowing us to discuss the
``next'' observation $j+1$ which may be some distance $l{j+1} - l_j$ away.  Note that we assume in
this notation that all replicates have been observed at the same set of genomic locations.

The basic statistical model is the following:
\begin{displaymath}
Y_{ij} = \beta_0(l_j) + \beta_1(l_j) X_j + \varepsilon_{ij}
\end{displaymath}
with $i$ representing subject, $l_j$ representing the $j$th location, $X_j$ is the covariate of interest (for
example $X_j=1$ for cases and $X_j=0$ otherwise), $\varepsilon_{ij}$ is measurement error, $\beta_0(l)$ is
a baseline function, and $\beta_1(l)$ is the parameter of interest, which is a function of
location.  We assume that $\beta_1(l)$ will be equal to zero over most of the genome, and we want to
identify stretched where $\beta_1(l) \neq 0$, which we call \emph{bumps}.

We want to share information between nearby locations, typically through some form of smoothing.


\section*{Creating clusters}

For many genomic applications the locations are clustered.  Each cluster is a distinct unit where
the model fitting will be done separately, and each cluster does not depend on the data, only on the
locations $l_j$.  Typically there is some maximal distance, and we do not want to smooth between
observations separated by more than this distance.  The choice of distance is very application dependent.

``Clusters'' are simply groups of locations such that two consecutive locations in the cluster are
separated by less than some distance \Rcode{maxGap}.  For genomic applications, the biggest possible
clusters are chromosomes.

The function \Rcode{clusterMaker} defines such grouping locations.

Example: We first generate an example of typical genomic locations
<<clustermakerdata>>=
pos <- list(pos1=seq(1,1000,35),
            pos2=seq(2001,3000,35),
            pos3=seq(1,1000,50))
chr <- rep(paste0("chr",c(1,1,2)), times = sapply(pos,length))
pos <- unlist(pos, use.names=FALSE)
@

Now we run the function to obtain the three clusters from the positions. We use the default gap of
300 base pairs (bps), i.e. any two points more than 300 bps away are put in a new cluster. Also,
locations from different chromosomes are separated.

<<clustermaker>>=
cl <- clusterMaker(chr, pos, maxGap = 300)
table(cl)
@

The output is an indexing variable telling us which cluster each location belongs to.  Locations on
different chromosomes are always on different clusters.


Note that data from the first chromosome has been split into two clusters:
<<clusterplot,fig=TRUE,width=6,height=3>>=
ind <- which(chr=="chr1")
plot(pos[ind], rep(1,length(ind)), col=cl[ind],
     xlab="locations", ylab="")
@

\section*{Breaking into segments}

The function \Rcode{getSegments} is used to find segments that are positive, near zero, and
negative.  Specifically we have a vector of numbers $\theta_j$ with each number associated with a
genomic location $l_j$ (thinks either test statistics or estimates of $\beta_i(l)$).  A segment is a
list of consecutive locations such that all $\theta_l$ in the segment are either ``positive'', ``near zero'' or
``negative''.  In order to define ``positive'' etc we need a \Rcode{cutoff} which is one number $L$ (in
which case ``near zero'' is $[-L,L]$) or two numbers $L,U$ (in which case ``near zero'' is $[L;
U]$).


Example: we are going to create a simulated $\beta_1(l)$ with a couple of real bumps.
<<simulatedbumps>>=
Indexes <- split(seq_along(cl), cl)
beta1 <- rep(0, length(pos))
for(i in seq(along=Indexes)){
    ind <- Indexes[[i]]
    x <- pos[ind]
    z <- scale(x, median(x), max(x)/12)
    beta1[ind] <- i*(-1)^(i+1)*pmax(1-abs(z)^3,0)^3 ##multiply by i to vary size
}
@

We now find bumps of this functions by

<<getSegments>>=
segs <- getSegments(beta1, cl, cutoff=0.05)
@

Now we can make, for example, a plot of all the positive bumps

<<plotSegments,fig=TRUE,width=7,height=4>>=
par(mfrow=c(1,2))
for(ind in segs$upIndex){
    index <- which(cl==cl[ind[1]])
    plot(pos[index], beta1[index],
         xlab=paste("position on", chr[ind[1]]),
         ylab="beta1")
    points(pos[ind], beta1[ind], pch=16, col=2)
    abline(h = 0.05, col = "blue")
}
@

This function is used by regionFinder which is described next.

\section*{regionFinder}

This function packages up the results of \Rcode{getSegments} into a table of regions with the
location and characteristics of bumps.
<<regionFinder>>=
tab <- regionFinder(beta1, chr, pos, cl, cutoff=0.05)
tab
@

In the plot in the preceding section we show two of these regions in red.

Note that \Rcode{regionFinder} and \Rcode{getSegments} do not really contain any statistical model.
All it does is finding regions based on segmenting a vector $\theta_j$ associated with genomic
locations $l_j$.

\section*{Bumphunting}

\Rcode{Bumphunter} is a more complicated function.  In addition to \Rcode{regionFinder} and
\Rcode{clusterMaker} it also implements a statistical model as well as permutation testing to assess
uncertainty.

We start by creating a simulated data set of 10 cases and 10 controls (recall that \Rcode{beta1} was
defined above).

<<simulationOfReps>>=
beta0 <- 3*sin(2*pi*pos/720)
X <- cbind(rep(1,20), rep(c(0,1), each=10))
error <- matrix(rnorm(20*length(beta1), 0, 1), ncol=20)
y <- t(X[,1])%x%beta0 + t(X[,2])%x%beta1 + error
@

Now we can run \Rcode{bumphunter}
<<bumphunter>>=
tab <- bumphunter(y, X, chr, pos, cl, cutoff=.5)
tab
names(tab)
tab$table
@

Briefly, the \Rcode{bumphunter} function fits a linear model for each location (like \Rcode{lmFit} from the
\Rpackage{limma} package), focusing on one specific column (coefficient) of the design matrix.  This
coefficient of interest is optionally smoothed.  Subsequently, a
permutation can be used to test is formed for this
specific coefficient.

The simplest way to use permutations to create a null distribution is to
set \Rcode{B}. If the number of samples is large this can be set to a
large number, such as 1000. Note that this will be slow and we have
therefore provided parallelization capabilities. In cases were the user
wants to define the permutations, for example cases in which all
possible permutations can be enumerated, these can be supplied via the \Rcode{permutation}
argument. 

Note that the function permits the matrix \Rcode{X} to have more than
two columns. This can be useful for those wanting to fit models that
try to adjust for confounders, such as age and sex. However, when
\Rcode{X} has columns other than those representing an intercept term and
the covariate of interest, the permutation test approach is not
recommended. The function will run but give a warning.
A method based on the bootstrap for linear models of Efron and
Tibshirani  \cite{bootstrap} may be more appropriate but this is not currently
implemented. 

\section*{Faster bumphunting with multiple cores}

\Rcode{bumphunter} can be speeded up by using multiple cores.  We use the \Rcode{foreach} package
which allows different parallel "back-ends" that will distribute the computation across multiple
cores in a single machine, or across multiple machines in a cluster.  The most straight-forward
usage, illustrated below, involves multiple cores on a single machine.  See the \Rcode{foreach}
documentation for more complex use cases, as well as the packages \Rcode{doParallel} and
\Rcode{doSNOW} (among others).  Finally, we use \Rcode{doRNG} to ensure reproducibility of setting
the seed within the parallel computations.

In order to use the \Rcode{foreach} package we need to register a backend, in this case a multicore
machine with 2 cores.

<<load-foreach>>=
library(doParallel)
registerDoParallel(cores = 2)
@

\Rcode{bumphunter} will now automatically use this backend

<<parallel-bumphunter>>=
tab <- bumphunter(y, X, chr, pos, cl, cutoff=.5, B=250, verbose = TRUE)
tab
@

\bibliographystyle{plain}
\bibliography{bumphunter}

\section*{Cleanup}

This is a cleanup step for the vignette on Windows; typically not needed for users.

<<closeConnetions,results=hide>>=
bumphunter:::foreachCleanup()
@
 
\section*{SessionInfo}

<<sessionInfo,results=tex,eval=TRUE,echo=FALSE>>=
toLatex(sessionInfo())
@

\end{document}

% Local Variables:
% eval: (add-hook 'LaTeX-mode-hook '(lambda () (if (string= (buffer-name) "bumphunter.Rnw") (setq fill-column 100))))
% LocalWords: LocalWords bisulfite methylation methylated CpG CpGs DMR
% LocalWords: DMRs differentially
% End:

\name{matchGenes}
\alias{matchGenes}
\title{
Find and annotate closest genes to genomic regions
}
\description{
Find and annotate closest genes to genomic regions
}
\usage{
matchGenes(x, subject, type = c("any", "fiveprime"), promoterDist = 2500, skipExons = FALSE, verbose = TRUE)
}
\arguments{
  \item{x}{
    An \code{IRanges} or \code{GenomicRanges} object, or a
    \code{data.frame} with columns for \code{start}, \code{end},
    and, optionally, \code{chr} or \code{seqnames}.
  }
  \item{subject}{
    An \code{GenomicRanges} object containing transcripts or genes that
    have been annotated by the function
    \code{\link{annotateTranscripts}}. 
  }
  \item{promoterDist}{
    Anything within this distance to the transcription start site (TSE)
    will be considered a promoter.
  }
  \item{type}{
    Should the distance be computed to any part of the transcript or the
    five prime end.
}
\item{skipExons}{
    Should the annotation of exons be skipped. Skipping this part makes the
    code slightly faster.
  }
  \item{verbose}{
    logical value. If 'TRUE', it writes out some messages
    indicating progress. If 'FALSE' nothing should be printed.
  }
}
\details{
 This function runs \code{\link[IRanges]{nearest}} and then annotates
 the the relationship between the region and the transcript/gene that is
 closest. Many details are provided on this relationship as described in
 the next section. 

}
\value{
  A data frame with one row for each query and with columns \code{c("name",
  "annotation", "description", "region", "distance", "subregion",
  "insideDistance", "exonnumber", "nexons", "UTR", "strand", "geneL", "codingL","Entrez","subhectHits")}.
  The first column is the _gene_ nearest the query, by virtue of it
  owning the transcript determined (or chosen by \code{nearest}) to be
  nearest the query. Note that  the nearest gene to a given
  query, in column 3, may not be unique and we arbitrarily chose one as
  done by default by \code{\link[IRanges]{nearest}}.
  
  The "distance" column is the distance from the query to the 5' end of the
  nearest transcript, so may be different from the distance computed by
  \code{nearest} to that transcript, as a range.

\item{name}{Symbol of nearest gene}
\item{annotation}{RefSeq ID}
\item{description}{a factor with levels \code{c("upstream", "promoter",
	"overlaps 5'", "inside intron", "inside exon", "covers exon(s)",
	"overlaps exon upstream", "overlaps exon downstream",
	"overlaps two exons", "overlaps 3'", "close to 3'", "downstream",
	"covers")}}
\item{region}{a factor with levels \code{c("upstream", "promoter", "overlaps 5'",
	"inside", "overlaps 3'", "close to 3'", "downstream", "covers")}}
\item{distance}{distance before 5' end of gene}
\item{subregion}{a factor with levels \code{c("inside intron", "inside exon",
	"covers exon(s)", "overlaps exon upstream", "overlaps exon downstream",
	"overlaps two exons")}}
\item{insideDistance}{distance past 5' end of gene}
\item{exonnumber}{which exon}
\item{nexons}{number of exons}
\item{UTR}{a factor with levels \code{c("inside transcription region", "5' UTR",
	"overlaps 5' UTR", "3'UTR", "overlaps 3'UTR", "covers transcription region")}}
\item{strand}{"+" or "-"}
\item{geneL}{the gene length}
\item{codingL}{the coding length}
\item{Entrez}{Entrez ID of closest gene}
\item{subjectHits}{Index in subject of hit}
}
\author{
Harris Jaffee, Peter Murakami and Rafael A. Irizarry
}
\seealso{
\code{annotateNearest}, \code{annotateTranscripts}
}
\examples{
\dontrun{
    islands=makeGRangesFromDataFrame(read.delim("http://rafalab.jhsph.edu/CGI/model-based-cpg-islands-hg19.txt")[1:100,])
    library("TxDb.Hsapiens.UCSC.hg19.knownGene")
    genes <- annotateTranscripts(TxDb.Hsapiens.UCSC.hg19.knownGene)
    tab<- matchGenes(islands,genes)
}
}

\name{annotateNearest}
\alias{annotateNearest}
\title{
Annotate the results of nearest
}
\description{
Annotate the results of nearest with more information about the type of match.
}
\usage{
annotateNearest(x, subject, annotate = TRUE, ...)
}
\arguments{
  \item{x}{
The query. An \code{IRanges} or \code{GenomicRanges} object, or a
    \code{data.frame} with columns for \code{start}, \code{end},
    and, optionally, \code{chr} or \code{seqnames}.
}
  \item{subject}{
The subject.  An \code{IRanges} or \code{GenomicRanges} object, or a
    \code{data.frame} with columns for \code{start}, \code{end},
    and, optionally, \code{chr} or \code{seqnames}.
}
  \item{annotate}{
Whether to annotate the result.
}
  \item{\dots}{
Arguments passed along to \code{nearest}. 
}
}
\details{
This function runs \code{\link[IRanges]{nearest}} and then annotates the
nearest hit. Note that the nearest subject range to a given query may not be
unique and we arbitrarily chose one as done by default by
\code{\link[IRanges]{nearest}}.
}
\value{
A data frame with columns \code{c("distance", "subjectHits", "type",
  "amountOverlap", "insideDistamce", "size1", "size2")} unless
  \code{annotate} is \code{FALSE}, in which case only the first two
  columns are returned as an integer matrix.

 \item{dist}{Signed distance to the nearest target. Queries downstream
  from (i.e. past) their nearest target are given a negative distance.}
\item{subjectHits}{The index of the nearest target.}
\item{type}{one of \code{c("inside", "cover", "disjoint", "overlap")}.}
\item{amountOverlap}{The width of the overlap region, if any.}
\item{insideDistance}{When a query is contained in its nearest target, the
  signed minimum of the two distances target-start-to-query-start and
  query-end-to-target-end. The former is taken positive, and the latter,
  which wins in ties, negative. \code{dist} will be \code{0} in this case.}
\item{size1}{equals \code{width(x)}.}
\item{size2}{equals \code{width(subject)}.}
}
\author{
Harris Jaffee, Peter Murakami and Rafael A. Irizarry
}

\seealso{
\code{\link[IRanges]{nearest}}, \code{\link{matchGenes}} 
}

\examples{
query <- GRanges(seqnames = 'chr1', IRanges(c(1, 4, 9), c(5, 7, 10)))
subject <- GRanges('chr1', IRanges(c(2, 2, 10), c(2, 3, 12)))
nearest(query, subject)
distanceToNearest(query, subject)

## showing 'cover' and 'disjoint', and 'amountOverlap'
annotateNearest(query, subject)

## showing 'inside' and 'insideDist', and 'amountOverlap'
annotateNearest(subject, query)
annotateNearest(GRanges('chr1', IRanges(3,3)), GRanges('chr1', IRanges(2,5)))
annotateNearest(GRanges('chr1', IRanges(3,4)), GRanges('chr1', IRanges(2,5)))
annotateNearest(GRanges('chr1', IRanges(4,4)), GRanges('chr1', IRanges(2,5)))

}

\name{clusterMaker}
\alias{clusterMaker}
\alias{boundedClusterMaker}
\title{
  Make clusters of genomic locations based on distance
}

\description{
  Genomic locations are grouped into clusters based on distance:
  locations that are close to each other are assigned to the same
  cluster. The operation is performed on each chromosome independently.
}

\usage{
clusterMaker(chr, pos, assumeSorted = FALSE, maxGap = 300)
boundedClusterMaker(chr, pos, assumeSorted = FALSE,
                    maxClusterWidth = 1500, maxGap = 500)
}

\arguments{
  \item{chr}{A vector representing chromosomes.  This is usually a
    character vector, but may be a factor or an integer.}
  \item{pos}{A numeric vector with genomic locations.}
  \item{assumeSorted}{This is a statement that the function may assume
    that the vector \code{pos} is sorted (within each \code{chr}).
    Allowing the function to make this assumption may increase the speed
    of the function slightly.} 
  \item{maxGap}{An integer. Genomic locations within maxGap from each other are
    placed into the same cluster.}
  \item{maxClusterWidth}{An integer. A cluster large than this width is
    broken into subclusters.}
}

\details{
  
  The main purpose of the function is to genomic location into clusters
  that are close enough to perform operations such as smoothing.  A
  genomic location is a combination of a chromosome (\code{chr}) and an
  integer position (\code{pos}).  Specifically, genomic intervals are
  not handled by this function.

  Each chromosome is clustered independently from each other.  Within
  each chromosome, clusters are formed in such a way that two positions
  belong to the same cluster if they are within \code{maxGap} of each
  other.
}

\value{
  A vector of integers to be interpreted as IDs for the clusters, such
  that two genomic positions with the same cluster ID is in the same
  cluster.  Each genomic position receives one integer ID.
}

\author{
  Rafael A. Irizarry, Hector Corrada Bravo
}

\examples{
N <- 1000
chr <- sample(1:5, N, replace=TRUE)
pos <- round(runif(N, 1, 10^5))
o <- order(chr, pos)
chr <- chr[o]
pos <- pos[o]
regionID <- clusterMaker(chr, pos)
regionID2 <- boundedClusterMaker(chr, pos)
}

\name{dummyData}
\alias{dummyData}
\title{
  Generate dummy data for use with bumphunter functions
}

\description{
  This function generates a small dummy dataset representing
  samples from two different groups (cases and controls) that is used in
  bumphunter examples.
}

\usage{
dummyData(n1 = 5, n2 = 5, sd = 0.2, l = 100, spacing = 100,
          clusterSpacing=1e5, numClusters=5)
}

\arguments{
  \item{n1}{Number of samples in group 1 (controls)}
  \item{n2}{Number of samples in group 2 (cases)}
  \item{sd}{Within group standard deviation to be used when simulating data}
  \item{l}{The number of genomic locations for which to simulate data}
  \item{spacing}{The average spacing between locations. The actual
    locations have a random component so the actual spacing will be
    non-uniform}
  \item{clusterSpacing}{The spacing between clusters. (Specifically, the
    spacing between the first location in each cluster.)} 
  \item{numClusters}{Divide the genomic locations into this number of
    clusters, each of which will contain locations spaced \code{spacing}
    bp apart.} 
}

\value{
  A list containing data that can be used with various bumphunter functions.   
  \item{mat}{ A simulated data matrix with rows representing genomic locations and
  columns representing samples.}
  \item{design}{ Design matrix with rows representing samples and columns
  representing covariates.}  
  \item{chr}{A character vector with the chromosomes of each location.}
  \item{pos}{A numeric vector representing the chromosomal position.}
  \item{cluster}{A vector representing the cluster of each locations}
  \item{n1}{Number of samples in cluster 1}
  \item{n2}{Number of samples in cluster 2}
}

\author{
  Martin J. Aryee
}


\examples{
dat <- dummyData()
names(dat)
head(dat$pos)
}

\name{runmedByCluster}
\alias{runmedByCluster}

\title{
Apply running median smoothing to values within each spatially-defined
cluster
}

\description{
Running median smoothing is applied independently to each cluster of
genomic locations. Locations within the same cluster are close together
to warrant smoothing across neighbouring locations. 
}

\usage{
runmedByCluster(y, x = NULL, cluster, weights = NULL, k = 5,
                endrule = "constant", verbose = TRUE) 
}

\arguments{
  \item{y}{A vector or matrix of values to be smoothed. If a matrix,
  each column represents a sample.}
  \item{x}{The genomic location of the values in y.}
  \item{cluster}{A vector indicating clusters of locations. A cluster is
    typically defined as a region that is small enough that it makes sense
    to smooth across neighbouring locations. Smoothing will only be applied
    within a cluster, not across locations from different clusters.} 
  \item{weights}{weights used by the smoother.}
  \item{k}{integer width of median window; must be odd. See \code{\link{runmed}}}
  \item{endrule}{character string indicating how the values at the
    beginning and the end (of the data) should be treated. See
    \code{\link{runmed}}.} 
  \item{verbose}{Boolean. Should progress be reported?}
}

\details{
This function is typically called by \code{\link{smoother}}, which is in
turn called by \code{\link{bumphunter}}.  
}

\value{
  \item{fitted }{The smoothed data values}
  \item{smoothed }{A boolean vector indicating whether a given position was smoothed}
  \item{spans }{The span used by the loess smoother. One per cluster.}
  \item{clusterL }{The number of locations in each cluster.}
  \item{smoother }{always set to \sQuote{runmed}.}
}

\author{
  Rafael A. Irizarry
}

\seealso{
  \code{\link{smoother}}, \code{\link{loessByCluster}}. Also see \code{\link{runmed}}.
}

\examples{
dat <- dummyData()
smoothed <- runmedByCluster(y=dat$mat[,1], cluster=dat$cluster,
                            k=5, endrule="constant")
}

\name{regionFinder}
\alias{regionFinder}
\title{
  Find non-zero regions in vector
}

\description{
  Find regions for which a numeric vector is above (or below) predefined thresholds.
}

\usage{
regionFinder(x, chr, pos, cluster = NULL, y = x, summary = mean,
             ind = seq(along = x), order = TRUE, oneTable = TRUE,
             maxGap = 300, cutoff=quantile(abs(x), 0.99),
             assumeSorted = FALSE, verbose = TRUE)
}

\arguments{
  \item{x}{A numeric vector.}
  \item{chr}{A character vector with the chromosomes of each location.}
  \item{pos}{A numeric vector representing the genomic location.}
  \item{cluster}{The clusters of locations that are to be analyzed
    together. In the case of microarrays, the cluster is many times
    supplied by the manufacturer. If not available the function
    \code{\link{clusterMaker}} can be used.}
  \item{y}{
    A numeric vector with same length as \code{x} containing values to
    be averaged for the region summary. See details for more.}
  \item{summary}{
    The function to be used to construct a summary of the \code{y}
    values for each region.}
  \item{ind}{
    an optional vector specifying a subset of observations to be used
    when finding regions.}
  \item{order}{
    if \code{TRUE} then the resulting tables are ordered
    based on area of each region. Area is defined as the absolute value
    of the summarized \code{y} times the number of features in the
    regions.}
  \item{oneTable}{
    if \code{TRUE} only one results table is returned. Otherwise, two
    tables are returned: one for the regions with positive values and
    one for the negative values.}
  \item{maxGap}{
    If cluster is not provided this number will be used to define clusters
    via the \code{\link{clusterMaker}} function.}
  \item{cutoff}{
    This argument is passed to \code{\link{getSegments}}.  It represents
    the upper (and optionally the lower) cutoff for \code{x}. }
  \item{assumeSorted}{
    This argument is passed to \code{\link{getSegments}} and
    \code{\link{clusterMaker}}.} 
  \item{verbose}{
    Should the function be verbose?}
}

\details{

  This function is used in the final steps of
  \code{\link{bumphunter}}. While \code{bumphunter} does many things,
  such as regression and permutation, \code{regionFinder} simply finds
  regions that are above a certain threshold (using
  \code{\link{getSegments}}) and summarizes them. The regions are found
  based on \code{x} and the summarized values are based on \code{y}
  (which by default equals \code{x}). The summary is used for the
  ranking so one might, for example, use t-tests to find regions but
  summarize using effect sizes.

}

\value{
  
 If \code{oneTable} is \code{FALSE} it returns two tables otherwise it
 returns one table. The rows of the table are regions. Information on
 the regions is included in the columns.

}

\author{
  Rafael A Irizarry
}

\seealso{

  \code{\link{bumphunter}} for the main usage of this function,
  \code{\link{clusterMaker}} for the typical input to the \code{cluster}
  argument and \code{\link{getSegments}} for a function used within
  \code{regionFinder}.

}

\examples{
x <- seq(1:1000)
y <- sin(8*pi*x/1000) + rnorm(1000, 0, 0.2)
chr <- rep(c(1,2), each=length(x)/2)
tab <- regionFinder(y, chr, x, cutoff=0.8)
print(tab[tab$L>10,])
}

\name{loessByCluster}
\alias{loessByCluster}
\title{
  Apply loess smoothing to values within each spatially-defined cluster.
}

\description{
Loess smoothing is applied independently to each cluster of genomic
locations. Locations within the same cluster are close together to
warrant smoothing across neighbouring locations.
}

\usage{
loessByCluster(y, x = NULL, cluster, weights = NULL, bpSpan = 1000,
               minNum = 7, minInSpan = 5, maxSpan = 1, verbose = TRUE)
}

\arguments{
  \item{y}{A vector or matrix of values to be smoothed. If a matrix,
    each column represents a sample.} 
  \item{x}{The genomic location of the values in y}
  \item{cluster}{A vector indicating clusters of locations. A cluster is
    typically defined as a region that is small enough that it makes sense
    to smooth across neighbouring locations. Smoothing will only be applied
    within a cluster, not across locations from different clusters.} 
  \item{weights}{weights used by the loess smoother}
  \item{bpSpan}{The span used when loess smoothing. (Expressed in base
    pairs.)}
  \item{minNum}{Clusters with fewer than \code{minNum} locations will
    not be smoothed}
  \item{minInSpan}{Only smooth the region if there are at least this
    many locations in the span.}
  \item{maxSpan}{The maximum span. Spans greater than this value will be
    capped.} 
  \item{verbose}{Boolean. Should progress be reported?}
}
\details{
This function is typically called by \code{\link{smoother}}, which is in
  turn called by \code{\link{bumphunter}}.
}
\value{
  \item{fitted }{The smoothed data values}
  \item{smoothed }{A boolean vector indicating whether a given position was smoothed}
  \item{smoother }{always set to \sQuote{loess}.}
}
\author{
Rafael A. Irizarry
}
\seealso{
\code{\link{smoother}}, \code{\link{runmedByCluster}}, \code{\link{locfitByCluster}}
}
\examples{
dat <- dummyData()
smoothed <- loessByCluster(y=dat$mat[,1], cluster=dat$cluster, bpSpan = 1000,
                         minNum=7, minInSpan=5, maxSpan=1)
}

\name{annotateTranscripts}
\alias{annotateTranscripts}
\title{
Annotate transcripts
}
\description{
Annotate transcripts
}
\usage{
annotateTranscripts(txdb, annotationPackage = NULL, by = c("tx","gene"), codingOnly=FALSE, verbose = TRUE, requireAnnotation = FALSE, mappingInfo = NULL, simplifyGeneID = FALSE)
}
\arguments{
  \item{txdb}{
    A \code{TxDb} database object such as \code{TxDb.Hsapiens.UCSC.hg19.knownGene}
  }
  \item{annotationPackage}{
    An annotation data package from which to obtain gene/transcript
    annotation. For example \code{org.Hs.eg.db}. If none is provided the
    function tries to infer it from \code{organism(txdb)} and if it can't it
    proceeds without annotation unless \code{ requireAnnotation = TRUE}.
  }
  \item{by}{
    Should we create a \code{GRanges} of transcripts (\code{tx}) or genes (\code{gene}).
  }
  \item{codingOnly}{
    Should we exclude all the non-coding transcripts.
  }
  
  \item{verbose}{
    logical value. If 'TRUE', it writes out some messages
    indicating progress. If 'FALSE' nothing should be printed.
  }
  \item{requireAnnotation}{
    logical value. If 'TRUE' function will stop if no annotation package
    is successfully loaded.
  }
  \item{mappingInfo}{
    a named list with elements 'column', 'keytype' and 'multiVals'. If specified
    this information will be used with \link[AnnotationDbi]{mapIds} when mapping
    the gene ids using \code{annotationPackage}. This is useful when working
    with a \code{txdb} object from ENSEMBL or GENCODE among other databases.
  }
  \item{simplifyGeneID}{
    logical value. If 'TRUE', gene ids will be shortened to before a dot is
    present in the id. This is useful for changing GENCODE gene ids to
    ENSEMBL ids.
  }
}
\details{
  This function prepares a \code{GRanges} for the \code{\link{matchGenes}}
  function. It adds information and in particular adds exons information
  to each gene/transcript.
}
\value{
  A \code{GRanges} object with an attribute \code{description} set to
  \code{annotatedTranscripts}. The following columns are added.
  \code{seqinfo} is the information returned by
  \code{\link[Seqinfo]{seqinfo}}, \code{CSS} is the coding region
  start, \code{CSE} is the coding region end, \code{Tx} is the transcript
  ID used in TxDb, \code{Entrez} is the Entrez ID, \code{Gene} is the gene
  symbol, \code{Refseq} is the RefSeq annotation, \code{Nexons} is the
  number of exons, \code{Exons} is an \code{IRanges} with the exon information.
}
\author{
  Harris Jaffee and Rafael A. Irizarry.
  'mappingInfo' and 'simplifyGeneID' contributed by Leonardo Collado-Torres.
}

\seealso{
  \code{\link{matchGenes}}
}
\examples{
\dontrun{
    library("TxDb.Hsapiens.UCSC.hg19.knownGene")
    genes <- annotateTranscripts(TxDb.Hsapiens.UCSC.hg19.knownGene)
    
    ##and to avoid guessing the annotation package:
    genes <- annotateTranscripts(TxDb.Hsapiens.UCSC.hg19.knownGene,annotation="org.Hs.eg.db")
    
}
}

\name{getSegments}
\alias{getSegments}
\title{
  Segment a vector into positive, zero, and negative regions
}
\description{
  Given two cutoffs, L and U, this function divides a numerical vector
  into contiguous parts that are above U, between L and U, and below L.
}
\usage{
getSegments(x, f = NULL, cutoff = quantile(abs(x), 0.99),
            assumeSorted = FALSE, verbose = FALSE)
}
\arguments{
  \item{x}{A numeric vector.}
  \item{f}{A factor used to pre-divide \code{x} into pieces. Each
    piece is then segmented based on the cutoff.  Setting this to
    \code{NULL} says that there is no pre-division.  Often,
    \code{\link{clusterMaker}} is used to define this factor.}
  \item{cutoff}{a numeric vector of length either 1 or 2.  If length is
    1, U (see details) will be cutoff and L will be -cutoff.  Otherwise
    it specifies L and U.  The function will furthermore always use the
    minimum of cutoff for L and the maximum for U.}
  \item{assumeSorted}{This is a statement that the function may assume
    that the vector \code{f} is sorted.  Allowing the function to make
    this assumption may increase the speed of the function slightly.}
  \item{verbose}{Should the function be verbose?}
}
\details{
  This function is used to find the indexes of the \sQuote{bumps} in functions
  such as \code{\link{bumphunter}}. 

  \code{x} is a numeric vector, which is converted into three levels
  depending on whether x>=U (\sQuote{up}), L<x<U (\sQuote{zero}) or x<=L
  (\sQuote{down}), with L and U coming from \code{cutoff}.  We assume
  that adjacent entries in \code{x} are next to each other in some
  sense.  Segments, consisting of consecutive indices into \code{x}
  (ie. values between 1 and \code{length(x)}), are formed such that all
  indices in the same segment belong to the same level of \code{f} and
  have the same discretized value of \code{x}.

  In other words, we can use \code{getSegments} to find runs of \code{x}
  belonging to the same level of \code{f} and with all of the values of
  x either above U, between L and U, or below L.
}

\value{
  A list with three components, each a list of indices.  Each component
  of these lists represents a segment and this segment is represented by a
  vector of indices into the original vectors \code{x} and \code{f}.
  \item{upIndex:}{a list with each entry an index of contiguous values
    in the same segment.  These segments have values of \code{x} above U.}
  \item{dnIndex:}{a list with each entry an index of contiguous values
    in the same segment.  These segments have values of \code{x} below L.}
  \item{zeroIndex:}{a list with each entry an index of contiguous
    values in the same segment. These segments have values of \code{x}
    between L and U.}
}

\author{
  Rafael A Irizarry and Kasper Daniel Hansen
}

\seealso{
  \code{\link{clusterMaker}}
}
\examples{
x <- 1:100
y <- sin(8*pi*x/100)
chr <- rep(1, length(x))
indexes <- getSegments(y, chr, cutoff=0.8)
plot(x, y, type="n")
for(i in 1:3){
   ind <- indexes[[i]]
   for(j in seq(along=ind)) {
      k <- ind[[j]]
      text(x[k], y[k], j, col=i)
   }
}
abline(h=c(-0.8,0.8))
}

\name{smoother}
\alias{smoother}
\title{
Smooth genomic profiles
}

\description{
Apply smoothing to values typically representing the difference between
two populations across genomic regions.
}

\usage{
smoother(y, x = NULL, cluster, weights = NULL, smoothFunction,
         verbose = TRUE, ...) 
}

\arguments{
  \item{y}{A vector or matrix of values to be smoothed. If a matrix,
  each column represents a sample.}
  \item{x}{The genomic location of the values in y}
  \item{cluster}{A vector indicating clusters of locations. A cluster
    is typically defined as a region that is small enough that it makes
    sense to smooth across neighbouring  locations. Smoothing will only be
    applied within a cluster, not across locations from different
    clusters} 
  \item{weights}{weights used by the smoother.}
  \item{smoothFunction}{A function to be used for smoothing the estimate
    of the genomic profile. Two functions are provided by the package:
    \code{loessByCluster} and \code{runmedByCluster}.}
  \item{verbose}{Boolean. Should progress be reported?}
  \item{\dots}{Further arguments to be passed to \code{smoothFunction}}
}

\details{
This function is typically called by bumphunter prior to identifying
candidate bump regions. Smoothing is carried out within regions defined
by the \code{cluster} argument.
}

\value{
  \item{fitted }{The smoothed data values}
  \item{smoothed }{A boolean vector indicating whether a given position was smoothed}
  \item{spans }{The span used by the loess smoother. One per cluster.}
  \item{clusterL }{The number of locations in each cluster.}
  \item{smoother }{The name of the smoother used}
}

\author{
Rafael A. Irizarry and Martin J. Aryee
}

\seealso{
\code{\link{loessByCluster}}, \code{\link{runmedByCluster}}
}

\examples{
dat <- dummyData()

# Enable parallelization
require(doParallel)
registerDoParallel(cores = 2)

## loessByCluster
smoothed <- smoother(y=dat$mat[,1], cluster=dat$cluster, smoothFunction=loessByCluster, 
                       bpSpan = 1000, minNum=7, minInSpan=5, maxSpan=1)
## runmedByCluster
smoothed <- smoother(y=dat$mat[,1], cluster=dat$cluster, smoothFunction=runmedByCluster, 
                     k=5, endrule="constant")
# cleanup, for Windows
bumphunter:::foreachCleanup()
}

\name{bumphunter}
\alias{bumphunter}
\alias{bumphunter-methods}
\alias{bumphunter,matrix-method}
\alias{bumphunterEngine}
\alias{print.bumps}
\title{
  Bumphunter
}
\description{
  Estimate regions for which a genomic profile deviates from its baseline
  value. Originally implemented to detect differentially methylated
  genomic regions between two populations.
  
}
\usage{
\S4method{bumphunter}{matrix}(object, design, chr=NULL, pos, cluster=NULL,coef=2, cutoff=NULL, pickCutoff = FALSE, pickCutoffQ = 0.99, maxGap=500, nullMethod=c("permutation","bootstrap"),smooth=FALSE,smoothFunction=locfitByCluster, useWeights=FALSE, B=ncol(permutations), permutations=NULL,verbose=TRUE, ...)

bumphunterEngine(mat, design, chr = NULL, pos, cluster = NULL, coef = 2, cutoff = NULL, pickCutoff = FALSE, pickCutoffQ = 0.99, maxGap = 500, nullMethod=c("permutation","bootstrap"), smooth = FALSE, smoothFunction = locfitByCluster, useWeights = FALSE, B=ncol(permutations), permutations=NULL, verbose = TRUE, ...)
\S3method{print}{bumps}(x, \ldots)
}

\arguments{
  \item{object}{An object of class matrix.}
  \item{x}{An object of class \code{bumps}.}
  \item{mat}{A matrix with rows representing genomic locations and
    columns representing samples.}
  \item{design}{Design matrix with rows representing samples and columns
    representing covariates. Regression is applied to each row of mat.}
  \item{chr}{A character vector with the chromosomes of each location.}
  \item{pos}{A numeric vector representing the chromosomal position.}
  \item{cluster}{The clusters of locations that are to be analyzed
    together. In the case of microarrays, the clusters are many times
    supplied by the manufacturer. If not available the function
    \code{\link{clusterMaker}} can be used to cluster nearby locations.}
  \item{coef}{An integer denoting the column of the design matrix
    containing the covariate of interest. The hunt for bumps will be
    only be done for the estimate of this coefficient.}
  \item{cutoff}{A numeric value. Values of the estimate of the genomic
    profile above the cutoff or below the negative of the cutoff will be
    used as candidate regions. It is possible to give two separate
    values (upper and lower bounds). If one value is given, the lower
    bound is minus the value.}
  \item{pickCutoff}{Should bumphunter attempt to pick a cutoff using the
    permutation distribution?}
  \item{pickCutoffQ}{The quantile used for picking the cutoff using the
    permutation distribution.}
  \item{maxGap}{
    If cluster is not provided this maximum location gap will be used to define cluster
    via the \code{\link{clusterMaker}} function.}
  \item{nullMethod}{
    Method used to generate null candidate regions, must be one of \sQuote{bootstrap} or
    \sQuote{permutation} (defaults to \sQuote{permutation}). However, if covariates in addition to the 
    outcome of interest are included in the design matrix
  (ncol(design)>2), the \sQuote{permutation} approach is not
  recommended. See vignette and original paper for more information.}
  \item{smooth}{
    A logical value. If TRUE the estimated profile will be smoothed with the
    smoother defined by \code{smoothFunction}}
  \item{smoothFunction}{
    A function to be used for smoothing the estimate of the genomic
    profile. Two functions are provided by the package: \code{loessByCluster}
    and \code{runmedByCluster}.}
  \item{useWeights}{
    A logical value. If \code{TRUE} then the standard errors of the
   point-wise estimates of the profile function will be used as weights
   in the loess smoother \code{loessByCluster}. If the
   \code{runmedByCluster} smoother is used this argument is ignored.}
 \item{B}{An integer denoting the number of resamples to use when computing
   null distributions. This defaults to 0. If \code{permutations} is
  supplied that defines the number of permutations/bootstraps and \code{B} is
  ignored.}
\item{permutations}{ is a matrix with columns providing indexes to be used to
  scramble the data and create a null distribution when
  \code{nullMethod} is set to  permutations. If the bootstrap approach is used this argument is
  ignored. If this matrix is not supplied and \code{B}>0 then
  these indexes are created using the function \code{sample}. }
\item{verbose}{
    logical value. If \code{TRUE}, it writes out some messages
    indicating progress. If \code{FALSE} nothing should be printed.
  }
  \item{\dots}{
    further arguments to be passed to the smoother functions.
}
}
\details{

This function performs the bumphunting approach described by Jaffe et
al. International Journal of Epidemiology (2012). The main output is a
table of candidate regions with permutation or bootstrap-based family-wide 
error rates (FWER) and p-values assigned.

The general idea is that for each genomic location we have a value for
several individuals. We also have covariates for each individual and
perform regression. This gives us one estimate of the coefficient of
interest (a common example is case versus control). These estimates are
then (optionally) smoothed. The smoothing occurs in clusters of
locations that are \sQuote{close enough}. This gives us an estimate of a
genomic profile that is 0 when uninteresting. We then take values above
(in absolute value) \code{cutoff} as candidate regions. Permutations can
then performed to create null distributions for the candidate
regions.

The simplest way to use permutations or bootstraps to create a null distribution is to
set \code{B}. If the number of samples is large this can be set to a
large number, such as 1000. Note that this will be slow and we have
therefore provided parallelization capabilities. In cases were the user
wants to define the permutations or bootstraps, for example cases in which all
possible permutations/boostraps can be enumerated, these can be supplied via the 
\code{permutations} argument.


Uncertainty is assessed via permutations or bootstraps. Each of the \code{B} 
permutations/bootstraps will produce an estimated \sQuote{null profile} from which we 
can define \sQuote{null candidate regions}. For each observed candidate region we
determine how many null regions are \sQuote{more extreme} (longer and
higher average value). The \sQuote{p.value} is the percent of candidate
regions obtained from the permutations/boostraps that are as extreme as the observed
region. These p-values should be interpreted with care as the
theoretical proporties are not well understood. The \sQuote{fwer} is
the proportion of permutations/bootstraps that had at least one region as extreme as
the observed region. We compute p.values and FWER for the area of the
regions (as opposed to length and value as a pair) as well.
Note that for cases with more than one covariate the permutation
approach is not generally recommended; the \code{nullMethod} argument will coerce
to \sQuote{bootstrap} in this scenario. See vignette and original paper for more information.


Parallelization is implemented through the foreach package.
}

\value{
An object of class \code{bumps} with the following components:
\item{tab}{The table with candidate regions and annotation for these.}
\item{coef}{The single loci coefficients.}
\item{fitted}{The estimated genomic profile used to determine the
  regions.}
\item{pvaluesMarginal}{marginal p-value for each genomic location.}
\item{null}{The null distribution.}
\item{algorithm}{details on the algorithm.}
}

\references{
Jaffe AE, Murakami P, Lee H, Leek JT, Fallin MD, Feinberg AP, Irizarry
RA (2012) Bump hunting to identify differentially methylated regions in
epigenetic epidemiology studies. International Journal of Epidemiology
41(1):200-9.
}

\author{
Rafael A. Irizarry, Martin J. Aryee, Kasper D. Hansen, and Shan Andrews.
}

\examples{
dat <- dummyData()
# Enable parallelization
require(doParallel)
registerDoParallel(cores = 2)
# Find bumps
bumps <- bumphunter(dat$mat, design=dat$design, chr=dat$chr, pos=dat$pos,
                    cluster=dat$cluster, coef=2, cutoff= 0.28, nullMethod="bootstrap",
                    smooth=TRUE, B=250, verbose=TRUE,
                    smoothFunction=loessByCluster)
bumps
# cleanup, for Windows
bumphunter:::foreachCleanup()
}

\docType{data}
\name{TT}
\alias{TT}
\title{Example data}
\format{A list with three components.
\describe{
\item{txdb }{  Has a TxDb example.}
\item{org }{ has an Org DB example.}
\item{transcripts }{ has example transcripts output.}
}}
\description{
Example data
}
\keyword{datasets}
